package jive.security;

//MD
/*************************************************************************
 *  Questa classe rappresenta un elemento  della Access Control List (ACL) 
 *  associata ad ogni tupla dello spazio di tuple. Tale ACL consente di 
 *  specificare gli agenti che hanno i diritti per leggere o eliminare
 *  la tupla a cui e' associata quella ACL.
 *************************************************************************/

import java.security.acl.Permission;
import java.security.acl.*;
import java.security.*;
import java.util.*;


public class AclEntryImpl 
		implements AclEntry,java.io.Serializable {

  // questo campo specifica l'identita' dell'agente a cui vengono 
  // concessi i permessi che sono indicati nel seguito
  private Principal id;
		  
  // questa e' la lista dei permessi, nella implementazione attuale
  // gli unici due permessi sono il diritto di lettura (read )e quello 
  // di cancellazione dallo spazio (take).
  private Vector permission_list = new Vector();
		  
  // il campo che segue specifica se i permessi sopra indicati sono 
  // concessi o negati.
  private boolean access;
	
  public AclEntryImpl(Principal I) {
	id = I; 
	// per default i permessi indicati sono concessi.
	access = true;
  }  
/*************************************************************************
 *  Questo metodo consente di aggiungere un permesso. Ritorna true se 
 *  viene aggiunto false in caso contrario (cioe' se c'e' gia').
 *************************************************************************/
  public boolean addPermission(Permission permission) {
	
	if ( permission_list.contains(permission) ) return false;
	permission_list.addElement(permission);
	return true;
  }  
/*************************************************************************
 *  Questo metodo consente di verificare se un permesso di quel tipo e' 
 *  gia' stato concesso.
 *************************************************************************/
  public boolean checkPermission(Permission permission) {
	 return permission_list.contains(permission);
  }  
  // non e' stata implementata 
  public Object clone() {return null;}  
/*************************************************************************
 *  Questo restituisce  l'identita' dell'agente a cui fanno riferimento i
 *  diritti.
 *************************************************************************/
  public Principal getPrincipal() {
	
	return  id;
  }  
 /*************************************************************************
 *  Questo metodo consente verificare se i diritti indicati sono negati e 
 *  non concessi.
 *************************************************************************/
  public boolean isNegative() {
	return (!access);
  }  
/*************************************************************************
 *  Questo metodo consente di elencare i permessi.
 *************************************************************************/
 public Enumeration permissions() {
	return permission_list.elements();
  }  
/*************************************************************************
 *  Questo metodo consente di rimuovere un permessso.
 *************************************************************************/
   public boolean removePermission(Permission permission) {
	return permission_list.removeElement(permission);
   }   
/*************************************************************************
 *  Questo metodo consente di specificare che i diritti indicati sono 
 *  negati e non concessi.
 *************************************************************************/
  public void setNegativePermissions() {
	access = false;
  }  
/*************************************************************************
 *  Questo metodo setta l'identita' dell'agente a cui fanno riferimento i
 *  diritti, se e' gia stata fissata non la cambia e restituisce false per 
 *  specificare che l'operazione non e' stata eseguita, altrimenti true.
 *************************************************************************/
  public boolean setPrincipal(Principal user) {
	
	if (id == null) { id = user; return true; }
	else return false;
  }  
 /*************************************************************************
 *  Questo metodo consente di ottenere una forma stampabile della classe.
 *************************************************************************/
  public String toString() {
	String Id = "null";
	if (id != null) Id = id.toString();
	String Permiss = "";
	Enumeration e = permission_list.elements();
	
	// per stampare tutti i permessi della lista e' sufficiente 
	// stamparne due diversi, siccome ve ne sono di soli due tipi,
	// quindi stampo il primo e l'ultimo.
	
	for(;e.hasMoreElements();)
	  Permiss+="+"+e.nextElement().toString();
	return 	"Agent:	"+Id+" Access: "+access+" Right: "+Permiss;
  }  
}