package jive.security;

//MD
/*************************************************************************
 *  Questa classe non e' altro che la Access Control List (ACL) che 
 *  consente di regolare gli accessi alle tuple dello spazio di tuple.
 *************************************************************************/

import java.util.*;
import java.security.acl.Permission;
import java.security.acl.*;
import java.security.*;

public class  AclImpl extends OwnerImpl 
implements  Acl {
  
		// nome della Access Control List
	private String Acl_name;
  
		// Access Control List
	private Vector Acl_list = new Vector();
  
	public  AclImpl(Principal Id) {
		super(Id);
	  }  

		// aggiunge un elemento alla Access Control List solo se 
		// il caller e' un proprietario.
public boolean addEntry(Principal caller, AclEntry entry)
	throws NotOwnerException 
	{
		if (!isOwner(caller)) throw new NotOwnerException();
		Enumeration A = Acl_list.elements();
		for(;A.hasMoreElements();) 
		{
			AclEntry AA = (AclEntry)A.nextElement();
				// se c'e' una tupla relativa allo stesso Principal
				// e sempre dello stesso tipo (divieto o concessione)
				// non viene inserita la nuova entry.
			if ( AA.getPrincipal().equals(entry.getPrincipal()) &&AA.isNegative() == entry.isNegative())
				return false;
		}
		Acl_list.addElement(entry);
		return true;
	}  

private Vector addPermissions(Vector add1, Enumeration add2) 
{
	for (;add2.hasMoreElements();)  
	{
		Permission p = (Permission) add2.nextElement();
		if ( !add1.contains(p) ) 
			add1.addElement(p) ;
	} 
	return add1;
}  


// viene verificato se il principal passato come parametro 
// gode del diritto permission ...
public boolean checkPermission (Principal principal,Permission permission) 
{
	if (isOwner(principal)) 
		return true;
	Enumeration A = getPermissions( principal );
	for(;A.hasMoreElements();) {
		Permission AA = (Permission)A.nextElement();
		if ( AA.equals(permission) ) 
			return true;
	}
	return false;
}  

// vengono enumerati gli elementi della ACL.
public Enumeration entries() {
	return Acl_list.elements();
}  

public String getName() {
	return Acl_name;
}  
  // GESTISCE ANCHE IL CASO DI PRESENZA
  // DI GRUPPI.
  
public Enumeration getPermissions(Principal user) {
	
	Vector userperm = new Vector();
	Vector neguserperm = new Vector();
	Vector groupperm = new Vector();
	Vector neggroupperm = new Vector();
	

	// scorro gli elementi della Access Control List
	Enumeration A = Acl_list.elements();
	for(;A.hasMoreElements();) {
	  AclEntry AA = (AclEntry)A.nextElement();
	  
	  // il principal nella lista di quelli che posseggono dei
	  // diritti potrebbe essere o un agente (istanza di 
	  // AgentIdentity) o un gruppo (istanza di AgentGroup)
	  // e cosi' lo user che esegue l'operazione:
	  //
	  // * si verifica normalmente se le identita'
	  //   previste nella lista e quella dello user
	  //   coincidono (siano esse entrambi gruppi o agenti) e
	  //   in tal caso si aggiungono  i diritti ammessi 
	  // * se l'identita' nella lista e' un gruppo, si deve
	  //   anche verificare se lo user appartiene ad un gruppo
	  //   e concedergli i diritti del gruppo.

	  if ( user.equals(AA.getPrincipal()) ) {
		if ( !AA.isNegative() ) addPermissions(userperm,AA.permissions());
		else addPermissions(neguserperm,AA.permissions());
	  }
	  if ( (AA.getPrincipal() instanceof AgentGroup) &&
		( ((AgentGroup)AA.getPrincipal()).isMember(user) )) {
		if ( !AA.isNegative() ) addPermissions(groupperm,AA.permissions());  
		else addPermissions(neggroupperm,AA.permissions());
	  }  
	}

	subPermissions(groupperm,neggroupperm.elements());

	addPermissions(userperm,groupperm.elements());

	subPermissions(userperm,neguserperm.elements());
 
	// ritorna tutti i diritti che possiede, non quelli che
	// gli sono stati negati o semplicemente non concessi.
	return userperm.elements();
  }  
  // rimuove un elemento dalla Access Control List.
  public boolean removeEntry(Principal caller, AclEntry entry)
	   throws NotOwnerException {
	 
	 if (!isOwner(caller)) throw new NotOwnerException();
	 Enumeration A = Acl_list.elements();
	 for(;A.hasMoreElements();) {
	   AclEntry AA = (AclEntry)A.nextElement();
	   // se c'e' una entry relativa allo stesso Principal
	   // e sempre dello stesso tipo (divieto o concessione)
	   // allora viene rimossa.
	   if ( AA.getPrincipal().equals(entry.getPrincipal()) &&
		AA.isNegative() == entry.isNegative() )
	   return Acl_list.removeElement(AA);
	 }
	 return false;
  }  
  // questo metodo assegna il nome all'Access Control List
 public void setName(Principal caller, String name)
	   throws NotOwnerException {
	 
	 if (isOwner(caller)) Acl_name = name;
	 else throw new NotOwnerException();
  }  
  private Vector subPermissions(Vector pos, Enumeration neg) {
	for (;neg.hasMoreElements();) {
	  pos.removeElement(neg.nextElement());
	}
	return pos;
  }  
  // questo metodo fornisce una forma stampabile della classe
  public String toString() {
	// aggiungo gli owners nella stringa
	String tmp = "Access Control List\n"+
	  super.toString()+ "\nAccess right:\n";
	Enumeration e = Acl_list.elements();
	for(;e.hasMoreElements();)
	  tmp+="* "+((AclEntry)e.nextElement()).toString()+"\n";
	return tmp;
  }  
}