package jive.security;

//MD
/*************************************************************************
 *  Questa classe consente di rappresentare il proprietario della tupla
 *  consentendo a quest'ultimo di rendere altri agenti propietari di questa,
 *  o di eliminare dei propietari.
 *************************************************************************/

import java.util.*;
import java.security.*;
import java.security.acl.*;

public class OwnerImpl 
implements Owner //,java.io.Serializable,Cloneable
{
  
  // il vettore che segue memorizza la lista dei propietari. 
  private Vector list = new Vector();
  private Principal creator;

  // il metodo costruttore riceve come parametro l'identita' del 
  // proprietario e creatore.
  public OwnerImpl(Principal owner) {

	list.addElement(owner);
	creator = owner;
  }  
/*************************************************************************
 *  Questo metodo consente di aggiungere l'agente "owner" alla lista. Se il 
 *  "caller" non e' tra i proprietari non viene permesso e una 
 *  NotOwnerException viene sollevata.
 *************************************************************************/
  public boolean addOwner(Principal caller, Principal owner)
	   throws NotOwnerException {
	 
	 if (list.contains(caller)) {
	   if (list.contains(owner)) return false;
	   else { 
	     list.addElement(owner);
	     return true;
	   }
	 } else throw new NotOwnerException();
  }  
  public Principal creator() {
	return creator;
  }  
/*************************************************************************
 *  Questo metodo consente di cancellare l'owner passato tra i parametri.
 *  Per quanto riguarda la NotOwnerException vale un discorso analogo a
 *  quello del metod precedente. La LastOwnerException viene sollevata se 
 *  e' rimasto un solo Owner.
 *************************************************************************/
  public boolean deleteOwner (Principal caller, Principal owner)
	   throws NotOwnerException, LastOwnerException {
	 
	 if (list.size()==1) throw new LastOwnerException();
	 if (owner.equals(creator)  // && !caller.equals(creator)
	     ) throw new NotOwnerException();
	 if (list.contains(caller)) return list.removeElement(owner);
	 else throw new NotOwnerException();
  }  
  public boolean isCreator (Principal c) {
	return creator.equals(c);
  }  
/*************************************************************************
 *  Questo metodo permette di verificare se un certo agente e' fra i 
 *  proprietari.
 *************************************************************************/
  public boolean isOwner (Principal owner) {
	return list.contains(owner);
  }  
/*************************************************************************
 *  Questo metodo consente di scorrere tutti i proprietari.
 *************************************************************************/
  public Enumeration owners() {
	return list.elements();
  }  
/*************************************************************************
 *  Questo metodo fornisce una forma stampabile della classe.
 *************************************************************************/
  public String toString() {
	String tmp ="tuple's owners:";
	Enumeration e = list.elements();
	for(;e.hasMoreElements();)
	  tmp +="\n* "+e.nextElement().toString();
	return tmp;
  }  
}