package jive.util;

//MD copiato da Java-2-go!
/**************************************************************************

	Copyright (c) 1996 The Regents of the University of California.
	All rights reserved.

	Permission is hereby granted, without written agreement and without
	license or royalty fees, to use, copy, modify, and distribute this
	software and its documentation for any purpose, provided that the
	above copyright notice and the following two paragraphs appear in all
	copies of this software.

	IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
	FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
	ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
	THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
	SUCH DAMAGE.

	THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
	MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
	PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
	CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
	ENHANCEMENTS, OR MODIFICATIONS.

   ------------

   $Id: Passer.java,v 0.95 1996/06/25 05:11:01 wli Exp wli $

   Class PASSER: a monitor between the comm interface and the
   thread that provides all the computations

   Provides a FIFO good for (default = 10) connection requests
   all requests are casted into 'Object's 

  
  			+--------------+
  			+ Class Loader |
  			+--------------+
  			       ^^ <--------- SOCKET
  			   +--------+
  			   | Passer |
  			   +--------+
  			       ^^ <--------- SOCKET
  		      +-----------------+
  		      | Traffic Monitor | 
  		      +-----------------+


**************************************************************************/

import java.util.*;

/*************************************************************************
 *  This class serves as a monitor and a data carrier between
 *  multiple threads. The monitor implements a FIFO queue.
 *************************************************************************/

public class Passer {

	Vector storage;
	int maximum_size;
	boolean served = false;

/*************************************************************************
 *  This method constructs the monitor and the FIFO. The FIFO has no
 *  limit in length.
 *************************************************************************/

	public Passer() {
		maximum_size = -1;
		storage = new Vector();
	}
/*************************************************************************
 *  This method constructs the monitor and the FIFO. The FIFO has a
 *  capacity of <em>capacity</em> items.
 *************************************************************************/

	public Passer(int capacity) {
		maximum_size = capacity;
		storage = new Vector(capacity);
	}
/*************************************************************************
 *  This method obtains the next data item waiting in the FIFO. If there
 *  are no item to be obtained, the calling process is blocked. The
 *  obtained item is then removed from the FIFO.
 *************************************************************************/

	public synchronized Object get() {

	// if FIFO is empty, then wait for more data
	while (storage.isEmpty()) {
	    try { wait(); } catch (InterruptedException e) {}
	}
	
	// returns the packet first in queue
	Object tmp = storage.elementAt(0);

	// remove the served packet from the queue
	storage.removeElementAt(0);

	// let others know I've processed this packet.
	notify();

	return tmp;
	}
	// uguale alla precedente ma non viene rimosso il messaggio
	// dal buffer
	public synchronized Object getAndNotRemove() {

	// if FIFO is empty, then wait for more data
	while (storage.isEmpty()) {
	    try { wait(); } catch (InterruptedException e) {}
	}
	
	// returns the packet first in queue
	Object tmp = storage.elementAt(0);

	// let others know I've processed this packet.
	notify();

	return tmp;
	}
/*************************************************************************
 *  This method puts a data item into the FIFO. If the FIFO has
 *  reached its maximum value, then the calling process is blocked
 *  until one or more items are removed from the FIFO.
 *************************************************************************/

	public synchronized void put(Object v) {

		// if there're no room for more requests, wait.
	if (maximum_size > -1) {
	    if (storage.size() == storage.capacity()) {
	        try { wait(); } catch (InterruptedException e) {}
	    }
		}

	// add the new packet to the end of the vector
	storage.addElement((Object) v);

	// let others know that I got more unserviced data
	notify();
	}
/*************************************************************************
 *  The two method read and writr are used by the methods read, write, and
 *  take of the class  Space, because these methods need to not be blocked
 *  if the storage id empty.
 *************************************************************************/

/*************************************************************************
 *  This method obtains the next data item waiting in the FIFO. If there
 *  are no item to be obtained, the calling process is not blocked and is 
 *  returned a pointer null. The item obtained item is then removed
 *  from the FIFO. The method notify() is called for the writers waiting 
 *  because the storage is full.
 *************************************************************************/

	public synchronized Object read() {

	// if FIFO is empty, then wait for more data
	if (storage.isEmpty()) return null;	
	
	// returns the packet first in queue
	Object tmp = storage.elementAt(0);

	// remove the served packet from the queue
	storage.removeElementAt(0);

	// let others know I've processed this packet.
	notify();

	return tmp;
	}
/*************************************************************************
 *  This method returns the size of the FIFO.
 *************************************************************************/

	public synchronized int size() {

	return storage.size();
	}
/*************************************************************************
 *  This method puts a data item into the FIFO. If the FIFO has
 *  reached its maximum value, then the calling process is blocked
 *  until one or more items are removed from the FIFO. This methods do 
 *  not call the notify() because the reader processes do not wait for an 
 *  item if the storage is empty. 
 *************************************************************************/

	public synchronized void write(Object v) {

		// if there're no room for more requests, wait.
	if (maximum_size > -1) {
	    if (storage.size() == storage.capacity()) {
	        try { wait(); } catch (InterruptedException e) {}
	    }
		}

	// add the new packet to the end of the vector
	storage.addElement((Object) v);

	// let others know that I got more unserviced data
	notify();

	}
}