package jive.javaSpace;

/*************************************************************************
 *  Questa classe permette di rappresentare la forma serializzata di un 
 *  oggetto e mette a disposizione una serie di metodi che operano su tale 
 *  forma.
 *************************************************************************/

import java.lang.reflect.*;
import java.io.*;
import jive.security.AclImpl;
import java.security.acl.Acl;
import java.security.Principal;
import java.util.*;
import jive.util.*;

public final class EntryRep implements Entry {

		// questo campo memorizza il SUID della classe corrente e di 
		// tutte le superclassi di entry
	private long[] cl_serial;

		// questo campo memorizza il SUID della reactionEntry (non necessario)
	private long[] re_cl_serial;

		// Questo e' il campo che contiene l'entry serializzata.
	private byte[][] obj_serial;

		//contiene i campi di reaction entry serializzata, ma non entry
	private byte[][] re_obj_serial;

		// questo campo memorizza un riferimento al Class Server dello spazio
	private static SpaceClassServer SCS = new SpaceClassServer();

		//usiamo questa variabile per distinguere le operazioni sulle tuple
		//del metaspazio da quelle sulle tuple dello spazio passivo
	private boolean meta = false;

		// class loader della classe dell'oggetto che e' stato serializzato
	ClassLoader RCL = null;

		// si potrebbe pensare anche ad un ClassLoader per ReactionEntry
	//Date date; // = new Date();

		// identita' dell'agente che ha "loccato" la tupla
	Principal lock_id = null;
	
		//ACL
	private AclImpl acl = null;
	
		// segnala se la tupla  "loccata" o meno
	private boolean lock = false;
	
	private long id;
	private long expires;
	
	private Entry rep; 


/*************************************************************************
 *  Questo e' il Metodo costruttore e costruisce la EntryRep a partire da
 *  un oggetto che implementa Entry  che 
 *  gli viene passato come parametro.
 *************************************************************************/

/**************************************************************************
 * questo metodo, come tutti quelli di questa classe,  diviso in due 
 * parti: una per le tuple dello spazio passivo, l'altra per le
 * metatuple
 *************************************************************************/


public EntryRep() {
}

public EntryRep(Entry e, boolean meta) throws IllegalArgumentException 
{

	if (meta == false) {
	 
		// Estraggo il class loader da "e". e sara' lo stesso class
		// loader dell'agente che crea la entry, e lo salvo in
		// una variabile.
	
	 	RCL = (ClassLoader)e.getClass().getClassLoader();

		/* 
		   System.out.println("### Il classloader della entry"+" serializzata ");
	 	   System.out.println(" e':"+RCL);
		*/

		// Se c'e' un classloader associato significa che la classe
		// non era presente nel file system locale.
		// Se invece per caricare la classe della tupla non si e'
		// fatto uso di nessun class loader, allora anche nella 
		// deserializzazione non ce ne sara' bisogno. Nell' affermare
		// cio' si suppone implicitamente che se la classe e' caricata da 
		// file system locale, allora anche le classi di tutti i campi
		// della tupla siano in tale file system (ipotesi ragionevole).
		
			// Stream temporaneo, di lunghezza pari a 1000 byte.
		ByteArrayOutputStream out = new ByteArrayOutputStream(1000);
		
		
		// Faccio uso di un metodo della classe Class per estrarre
		// i campi come nome e tipo. Una volta individuati i campi
		// e' semplice recuperare gli oggetti corrispondenti e poi 
		// serializzarli. Tale metodo restituisce un array di oggetti
		// istanze della classe Fields (tale classe e' nel package 
		// reflect).

		try {
			DataOutputStream data = new DataOutputStream(out);
			ObjectOutputStream output = new ObjectOutputStream(out);
			
			Class rif = ((Object)e).getClass();
			Field[] attrib = rif.getFields();

			/*
			int numeroCampi= Array.getLength(attrib);
			for(k=1;k<=numeroCampi;k++)
				System.out.println("campo n. "+k+": "+attrib[k-1]);
			*/

			Class c = Class.forName("java.lang.Object");
			Vector SUIDs = new Vector();
			
			// nella entry memorizzo solo il SUID della classe e delle
			// sue superclassi, che sono necessari nel matching.
			// N.B. Prima di serializzare gli oggetti memorizzo anche la 
			// classe della tupla in forma serializzata nello space class
			// server, per potere in  seguito ripristinare l'oggetto.
		
				
				//il ciclo seguente percorre le classi risalendo fino a Object
	
			while(!((Object)c).equals(rif)) 
			{
					//trova il descrittore della classe rif
				ObjectStreamClass OSC = ObjectStreamClass.lookup(rif);
					// estraggo il SUID ...
				Long SUID = new Long(OSC.getSerialVersionUID());
				SUIDs.addElement(SUID);
		
				ByteArrayOutputStream out_cl = new ByteArrayOutputStream(500);
				ObjectOutputStream output_cl = new ObjectOutputStream(out_cl);
				output_cl.writeObject((Object)rif);

				String tmp = e.getClass().toString();
				SCS.addClass(SUID.longValue(), tmp.substring(tmp.lastIndexOf(" ")+1),out_cl.toByteArray());
 				rif = rif.getSuperclass();
			}

				//cl_serial  un array il cui numero di elementi  pari al numero di passi
				//che il while precedente ha impiegato per arrivare a Object
				// trasformo il vettore SUIDs in un array

			cl_serial = new long[SUIDs.size()];

			Enumeration E = SUIDs.elements();
			for( int i = 0; E.hasMoreElements(); i++)  {
				cl_serial[i] = ((Long)E.nextElement()).longValue();
				//System.out.println("elemento "+i+" di cl_serial: "+cl_serial[i]);
			}
	
				/*****************************
				 * qui comincia la serializzazione
				 **********************************/
	
				// ora serializzo i campi ...
			output = new ObjectOutputStream(out);
			output.flush();
			out.reset();
		
				//obj_serial  un array di byte, uno per ogni campo
				//ogni byte rappresenta la forma serializzata di un campo
			obj_serial = new byte[Array.getLength(attrib)][];
		
			for(int i = 0; i < Array.getLength(attrib); i++) {
				Object obj = attrib[i].get(e);
				output = new ObjectOutputStream(out);
				output.writeObject(obj);
				obj_serial[i] = out.toByteArray();
				output.flush();
				out.reset();
			}

				/************************************
				 * la serializzazione e' terminata
				 ***********************************/

	      	try { 
				acl = (AclImpl) e.getAcl();
			} 
			catch (NullPointerException ex){
				System.err.println("non c' il campo acl:lo creo di default" + ex);
				acl = null;
			}

  	  	} //fine del primo try

		catch (IllegalAccessException ex){
			System.err.println("The object is inaccessible(1):"+ex);
		}

	  	catch (ClassNotFoundException ex){
			System.out.println("Class not found section number (1):"+ex);
		} 		

		/*
		catch (NoSuchFieldException ex)
		{System.out.println("No such field exception (1):"+ex);} 		
		*/

		catch (IOException ex){
			System.err.println("IOException in section number (1):"+ex);}
		}

/******************************************
 * da qui in poi e' la parte meta del costruttore: speculare
 ******************************************/ 

	else  {     
			// se meta  true e si tratta di ReactionEntry
	 		// Estraggo il class loader da "e"  
			//  e sara' lo stesso class
			// loader dell'agente che crea la reactionentry, e lo salvo in
			// una variabile.
	 
		RCL = (ClassLoader)e.getClass().getClassLoader();

			/*
			System.out.println("Il classloader della rentry"+" serializzata ");
			System.out.println(" e':"+RCL);
			*/

		//Manfredini:non sono cos sicura che nel caso della reactionentry sia ragionevole
		//sarebbe meglio estrarre comunque il classLoader remoto!
		
		// Se c'e' un classloader associato significa che la classe
		// non era presente nel file system locale.
		// Se invece per caricare la classe della tupla non si e'
		// fatto uso di nessun class loader, allora anche nella 
		// deserializzazione non ce ne sara' bisogno. Nell' affermare
		// cio' si suppone implicitamente che se la classe e' caricata da 
		// file system locale, allora anche le classi di tutti i campi
		// della tupla siano in tale file system (ipotesi ragionevole).
		
				// Stream temporaneo, di lunghezza pari a 1000 byte.
		ByteArrayOutputStream out = new ByteArrayOutputStream(1000);
		
		// Faccio uso di un metodo della classe  Class per estrarre
		// i campi come nome e tipo. Una volta individuati i campi
		// e' semplice recuperare gli oggetti corrispondenti e poi 
		// serializzarli. Tale metodo restituisce un array di oggetti
		// istanze della classe Fields (tale classe e' nel package 
		// reflect).
		
		try {
			DataOutputStream data = new DataOutputStream(out);
			ObjectOutputStream output = new ObjectOutputStream(out);
	
			Class rif = ((Object)e).getClass();
			Field attrib[] = rif.getFields();
			Class c = Class.forName("java.lang.Object");
			Vector SUIDs = new Vector();
			
		// nella entry memorizzo solo il SUID della classe e delle
		// sue superclassi, che sono necessari nel matching.
		// N.B. Prima di serializzare gli oggetti memorizzo anche la 
		// classe della tupla in forma serializzata nello space class
		// server, per potere in  seguito ripristinare l'oggetto.
		
			while(!((Object)c).equals(rif)) 
			{
			
				ObjectStreamClass OSC =	ObjectStreamClass.lookup(rif);
					// estraggo il SUID ...
				Long SUID = new Long(OSC.getSerialVersionUID());
				SUIDs.addElement(SUID);
		
				ByteArrayOutputStream out_cl = new ByteArrayOutputStream(500);
				ObjectOutputStream output_cl = new ObjectOutputStream(out_cl);

				output_cl.writeObject((Object)rif);
				String tmp = e.getClass().toString();
				SCS.addClass(SUID.longValue(),tmp.substring(tmp.lastIndexOf(" ")+1),
					out_cl.toByteArray());
 					rif = rif.getSuperclass();
			}
			
	
				// trasformo il vettore SUIDs in un array
				// si tratta di quello della reactionentry
			re_cl_serial = new long[SUIDs.size()];
			Enumeration E = SUIDs.elements();
		
			for( int i = 0; E.hasMoreElements(); i++) 
				re_cl_serial[i] = ((Long)E.nextElement()).longValue();
		
				// ora serializzo i campi ...
			output = new ObjectOutputStream(out);
			output.flush();
			out.reset();
				
				//questo array corrisponde a quello obj_serial
				//della parte meta=false
			re_obj_serial = new byte[Array.getLength(attrib)][];
				//nel primo campo metto la entry serializzata
		
			for(int i = 1; i < Array.getLength(attrib); i++) {
				Object obj = attrib[i].get(e);
				output = new ObjectOutputStream(out);
				output.writeObject(obj);
				re_obj_serial[i-1] = out.toByteArray();
					//System.out.println(re_obj_serial[i-1]);
				output.flush();
				out.reset();
			}
						
		 
			Entry re = ((ReactionEntry)e).getEntry();
			ByteArrayOutputStream rout = new ByteArrayOutputStream(1000);
		
		
		// Faccio uso di un metodo della classe  Class per estrarre
		// i campi come nome e tipo. Una volta individuati i campi
		// e' semplice recuperare gli oggetti corrispondenti e poi 
		// serializzarli. Tale metodo restituisce un array di oggetti
		// istanze della classe Fields (tale classe e' nel package 
		// reflect).
		
			DataOutputStream rdata = new DataOutputStream(rout);
			ObjectOutputStream routput = new ObjectOutputStream(rout);
	
			Class rrif = ((Object)re).getClass(); //FOND: QUI C'ERA e E NON re
			Field rattrib[] = rrif.getFields();
			Class rc = Class.forName("java.lang.Object");
			Vector rSUIDs = new Vector();
			
		// nella entry memorizzo solo il SUID della classe e delle
		// sue superclassi, che sono necessari nel matching.
		// N.B. Prima di serializzare gli oggetti memorizzo anche la 
		// classe della tupla in forma serializzata nello space class
		// server, per potere in  seguito ripristinare l'oggetto.
		
			while(!((Object)rc).equals(rrif)) {
				ObjectStreamClass rOSC =ObjectStreamClass.lookup(rrif);
					// estraggo il SUID ...
				Long rSUID = new Long(rOSC.getSerialVersionUID());
				rSUIDs.addElement(rSUID);
		
				ByteArrayOutputStream rout_cl = new ByteArrayOutputStream(500);
				ObjectOutputStream routput_cl = new ObjectOutputStream(rout_cl);
				routput_cl.writeObject((Object)rrif);
				String rtmp = re.getClass().toString();	//FOND: QUI C'ERA e E NON re
				SCS.addClass(rSUID.longValue(), rtmp.substring(rtmp.lastIndexOf(" ")+1),
					rout_cl.toByteArray());
				rrif = rrif.getSuperclass();
			}
		
				// trasformo il vettore SUIDs in un array
			cl_serial = new long[rSUIDs.size()];
			Enumeration rE = rSUIDs.elements();
			for( int i = 0; rE.hasMoreElements(); i++) 
				cl_serial[i] = ((Long)rE.nextElement()).longValue();
		
				// ora serializzo i campi ...
			routput = new ObjectOutputStream(rout);
			routput.flush();
			rout.reset();
		
			obj_serial = new byte[Array.getLength(rattrib)][];

			for(int i = 0; i < Array.getLength(rattrib); i++) {
				Object robj = rattrib[i].get(re);
				routput = new ObjectOutputStream(rout);
				routput.writeObject(robj);
				obj_serial[i] = rout.toByteArray();
				routput.flush();
				rout.reset();
			}

      		try { 
		        acl = (AclImpl) e.getAcl();
			} 

			catch (NullPointerException ex)
			{
				System.err.println("non c' il campo acl:lo creo di default" + ex);
				acl = null;
			}// in realt dovrei controllare il default

		} 
		catch (IllegalAccessException ex)
   	   		{System.err.println("The object is inaccessible(1):"+ex);}
		catch (ClassNotFoundException ex)
			{System.out.println("Class not found section number (1):"+ex);} 		

		catch (IOException ex)
			{System.err.println("IOException in section number (1):"+ex);}
		
	}//end else
	
}

/***********************************************************************/

public EntryRep(Entry e) throws IllegalArgumentException {
	this(e, false);
}


/*************************************************************************
 *  Questo metodo ritorna l'oggetto Entry corrispondente all'oggetto
 *  EntryRep corrente e nella deserializzazione viene utilizzato il Class 
 *  Loader di chi legge la tupla.
 *************************************************************************/

public Entry entry(boolean meta) //   throw new UnusableEntryException();   
{

	if (meta == false) {
  		Object obj = null;
		Class rif = null;
  	
		try {
			ObjectInputStream in_cl;
			for(int i = 0; i < cl_serial.length; i++) {
					// ottengo la classe serializzata dallo Space
					// Class Server
				try {
					in_cl = new ObjectInputStream( new ByteArrayInputStream(SCS.getClass(cl_serial[i])));
			
						// la classe viene automaticamente deserializzata
						// utilizzando il ClassLoader del lettore.
					rif = (Class)in_cl.readObject();
						//System.out.println("sono la entry: la classe deserializzato dovrebbe essere: "+rif);
					obj = rif.newInstance();
					break;	
						// se nessuna ClassNotFoundException si termina il
						// ciclo.
				} 
				catch(ClassNotFoundException exp)
		  		{
					System.out.println("Il ClassLoader del lettore non trova la classe: "+
					SCS.getName(cl_serial[i]));
					return null;
				}
		 	} 	
		
				// estraggo gli attributi dalla classe
			Field attrib[] = rif.getFields();
		
			/*
			int numeroCampi= Array.getLength(attrib);
			int k=1;
			for(k=1;k<=numeroCampi;k++)
				System.out.println("campo n. "+k+": "+attrib[k-1]);
			*/

			/*		
			System.out.println("### Il class loader"+" della classe dell'oggetto "); 
			System.out.println("    deserializzato e': "+rif.getClassLoader());
			*/

			// deserializzo i campi e li assegno alla nuova istanza
			// dell'oggetto che e' stata creata.

			ByteArrayInputStream in = null;
			ObjectInputStream input = null;
		
			for(int i = 0; i < Array.getLength(attrib); i++) {
				in = new ByteArrayInputStream(obj_serial[i]);		
				in.reset();
				input = new ObjectInputStream(in);
				Object objj = input.readObject();
				attrib[i].set(obj, objj);
			}
		  			
			// Dopo avere letto tutti i campi, leggo dalla stream di byte
			// anche i due oggetti che rappresentano la Data di scrittura
			// e la Access Control List.
		
			//((Entry)obj).set(acl);tale informazione deve essere solo della EntryRe
		 
		} 
		catch (InstantiationException e)
		  	{System.err.println("The class is abstract (2):"+e);}
	  	catch (IllegalAccessException e)
		  	{System.err.println("The object is inaccessible(2):"+e);}
	  	catch (ClassNotFoundException ex)
			{System.err.println("Class no found section number (2):"+ex);}
	  	catch (IOException ex)
			{System.err.println("IOException in section number (2):"+ex);}

		return (Entry)obj;
	}

/******************************************
 * da qui in poi e' la parte meta del metodo Entry: speculare
 ******************************************/ 

	else //meta true
 	{
	  	Object obj = null;
  		Class rif = null;

		try {
			ObjectInputStream in_cl;
			for(int i = 0; i < re_cl_serial.length; i++) {
					// ottengo la classe serializzata dallo Space
					// Class Server

				try {
					in_cl = new ObjectInputStream( new ByteArrayInputStream(SCS.getClass(re_cl_serial[i])));
			
						// la classe viene automaticamente deserializzata
						// utilizzando il ClassLoader del lettore.
					rif = (Class)in_cl.readObject();
					obj = rif.newInstance();
						//System.out.println("primo step"+rif);
	
					break;
						// se nessuna ClassNotFoundException si termina il
						// ciclo.
				} 

				catch(ClassNotFoundException exp){
					System.out.println("###  Il ClassLoader del "+"lettore non trova la classe: "+
						SCS.getName(re_cl_serial[i]));
					return null;
				}
	 		} 	
		 
		
				// estraggo gli attributi dalla classe
			Field attrib[] = rif.getFields();
		
				// deserializzo i campi e li assegno alla nuova istanza
				// dell'oggetto che e' stata creata.
			ByteArrayInputStream in = null;
			ObjectInputStream input = null;
		
			for(int i = 1; i < Array.getLength(attrib); i++) 
			{
				in = new ByteArrayInputStream(re_obj_serial[i-1]);		
				in.reset();
				input = new ObjectInputStream(in);
  
				Object objj = input.readObject();

			/*
			System.out.println(objj);
			System.out.println(obj.getClass().toString());
			System.out.println(attrib[i].toString());
			*/
	
				attrib[i].set(obj, objj);

				EntryRep nested_entry = new EntryRep();
				nested_entry.obj_serial = this.obj_serial;
				nested_entry.cl_serial = this.cl_serial;
				nested_entry.SCS = this.SCS;
				nested_entry.RCL = this.RCL;
				attrib[0].set(obj, nested_entry.entry(false));
			}
		  			
		// Dopo avere letto tutti i campi, leggo dalla stream di byte
		// anche i due oggetti che rappresentano la Data di scrittura
		// e la Access Control List.
		
			((Entry)obj).setAcl(acl);
		 
		} 
		catch (InstantiationException e)
	  		{System.err.println("The class is abstract (2):"+e);}
	  	catch (IllegalAccessException e)
			{System.err.println("The object is inaccessible(2):"+e);}
	  	catch (ClassNotFoundException ex)
			{System.err.println("Class no found section number (2):"+ex);}
	  	catch (IOException ex)
			{System.err.println("IOException in section number (2):"+ex);}

  	return (Entry)obj;
	}
}


/*************************************************************************
 *  Questo metodo ritorna l'oggetto Entry corrispondente all'oggetto
 *  EntryRep corrente e nella deserializzazione viene utilizzato il Class 
 *  Loader di chi legge la tupla.
 *************************************************************************/

public Entry entry() 
{
	Entry rit = this.entry(false);
	return rit;
}


/*************************************************************************
 *  Questo metodo ritorna l'oggetto Entry corrispondente all'oggetto
 *  EntryRep corrente e nella deserializzazione viene utilizzato il Class 
 *  Loader dello scrittore della tupla.
 *************************************************************************/

	//a questo metodo non ho messo mano in nessun modo
public Entry entrySpecial() 
	//   throw new UnusableEntryException();   
{
  	Object obj = null;
  	Class rif;
 
	try { 	
	  // ottengo il nome della classe della tupla dallo space 
	  // class server...
	  // poi la faccio caricare dal class loader della entry se ne 
		  // aveva associato uno oppure dal class loader di Java
	if (RCL != null)
	   rif = (Class)RCL.loadClass(SCS.getName(cl_serial[0]));
	  else rif = Class.forName(SCS.getName(cl_serial[0]));
	  obj = rif.newInstance();
 
	  // estraggo gli attributi dalla classe
	  Field attrib[] = rif.getFields();
	  
	  // System.out.println("### Il class loader"+" della classe dell'oggetto che"); 
	  // System.out.println("    e' stato serializzato e':  "+rif.getClassLoader());
	  
	  ByteArrayInputStream in = null;
	  ObjectInputStream input = null;
	  	
	  for(int i = 0; i < Array.getLength(attrib); i++) {
	    in = new ByteArrayInputStream(obj_serial[i]);		
	    in.reset();
	    input = new ObjectInputStream(in);
	    Object objj = input.readObject();
	    // if (objj != null)
		//	System.out.println("### Il class loader dell'oggetto"+
		//	" del campo: "+i+" e' "+objj.getClass().getClassLoader());
		    attrib[i].set(obj, objj);
	  }
		  			
		// Dopo avere letto tutti i campi, leggo dalla stream di byte
		// anche i due oggetti che rappresentano la Data di scrittura
		// e la Access Control List.
	
	        //((Entry)obj).setAcl(acl);
		 
	} catch (InstantiationException e)
	  	{System.err.println("The class is abstract (2):"+e);}
	  	catch (IllegalAccessException e)
 	{System.err.println("The object is inaccessible(2):"+e);}
	  	catch (ClassNotFoundException ex)
	{System.err.println("Class no found section number (2):"+ex);}
	catch (IOException ex)
	  	{System.err.println("IOException in section number (2):"+ex);}

  	return (Entry)obj;
	}


/*************************************************************************
 *  Questo  metodo ritorna true se l'EntryRep che gli viene passato 
 *  coincide con quello a cui corrisponde il metodo che viene invocato.
 *  In particolare il confronto consiste nel verificare che gli array di
 *  byte siano perfettamente coincidenti e i SUID siano gli stessi.
 *************************************************************************/

public boolean equals(EntryRep t)
{
	try {
		if ( Array.getLength(cl_serial) != 
			Array.getLength(t.cl_serial) ) return false;		
	  	
		for( int m = 0; m < Array.getLength(cl_serial); m++) {
			if( cl_serial[m] != t.cl_serial[m] ) 
				return false;
		}
		
		for( int m = 0; m < (Array.getLength(obj_serial)); m++) {
			int length = Array.getLength(obj_serial[m]);
			if (length != Array.getLength(t.obj_serial[m]) ) 
				return false;
	   		for( int l = 0; l < length ; l++) 
				if (obj_serial[m][l] != t.obj_serial[m][l]) 
					return false;
		}
	  
		return true;
	  
	}
	catch (NullPointerException e)
		{System.err.println("Error the object is null (4):"+e);} 
	catch (Exception e)
		{System.err.println("Exception in section number (4):"+e);}

	return false;
  }  


/*************************************************************************
 *  Questo  metodo ritorna la Access Control List, senza dovere 
 *  deserializzare l'intera entry.
 *************************************************************************/
public Acl getAcl()
{
	return acl;
}  


/*************************************************************************
 *  Questo  metodo ritorna true se l'EntryRep corrente e' locked, false 
 *  in caso contrario. Nel caso in cui la tupla era stata "loccata" dallo
 *  stesso agente specificato da who allora viene comunque ritornato false.
 *  N.B: con "stesso agente" si intende un controllo che verifica che i 
 *  due oggetti che specificano l'identita' siano la stessa istanza.
 *************************************************************************/

public synchronized boolean isLocked(Principal who) 
{
		// se la tupla non e' locked ritorno false
	if (lock == false) 
		return false;
		
		// se lo e', pero' l'identita' di chi l'ha loccata
		// e' la stessa di chiede se e' bloccata, allora 
		// ritorno false.
	else 
		if ( who == lock_id ) 
			return false;
		else 
			return true;
 } 

 
/*************************************************************************
 *  Questo  metodo esegue il locking di tupla e ritorna true se la tupla 
 *  viene "loccata" e false se e' gia' stata loccata.
 *************************************************************************/

public synchronized boolean lock(Principal who) 
{
	if (!lock) {
		  // se la tupla non e' locked viene 
		  // "loccata" e viene ritornato true.
		lock = true;
		lock_id = who;
		return true;	
	} 
	else 
		return false;
}  


/*************************************************************************
 *  Questo metodo esegue il match tra la EntryRep corrente e quella che 
 *  gli viene passata. E'da notare che il matching non prende in esame la
 *  Access conrol list e la data (tali controlli sono fatti a piu' alto
 *  livello).
 *************************************************************************/

public boolean matches(EntryRep entry, boolean meta) 
{

	// Definisco due variabili Class in cui mettiamo le due classi 
	// Definisco due stream che leggono sui due array di byte
	// N.B. le variabili che si riferiscono all'oggetto di cui si e'
	// invocato il metodo sono indicate con '1' le altre con '2'
   	// N.B.: SUPPONGO CHE L'OGGETTO DI CUI VIENE CHIAMATO IL METODO SIA
	//  IL TEMPLATE!

	//System.out.println("sono la matches: il mio parametro EntryRep e' "+entry);

	if (meta == false) {
	  	boolean trovato = false;
	    	Object objj1,objj2;
		int h = 0;
	
	    	//verifichiamo che le due entry di cui stiamo facendo la match siano
		//appartenenti alla stessa classe
	    		
	      while( h < Array.getLength(entry.cl_serial)) { 
			if(cl_serial[0] == entry.cl_serial[h++])
		  	trovato = true; 

			/*
			System.out.println("le classi ci permettono di confrontare i valori");
			System.out.println("matches ciclo "+(h-1)+" confronto "+entry.cl_serial[h-1]+" e "+cl_serial[0]);
			Class cs = SCS.getClass(entry.c1_serial[h]);
			System.out.println( cs.toString() );
			*/

	      }
	    	
		//appertengono alla stessa classe, e quindi possiamo confrontare i valori

		if (trovato) {
	      	int l1,l2;
			//System.out.println("trovato");
		      int length=Array.getLength(obj_serial);  
		      for(int i = 0; i < length ; i++) {

				/*
				System.out.println("Il numero di campi dell'oggetto e':"+(Array.getLength(obj_serial)));
				System.out.println("mio oggetto");
				System.out.println("classe: "+obj_serial[i].getClass());
				System.out.println("valore: "+obj_serial[i].toString());
				System.out.println("oggetto entry");
				System.out.println("classe: "+entry.obj_serial[i].getClass());
				System.out.println("valore: "+entry.obj_serial[i].toString());
				*/

				l1 = Array.getLength(obj_serial[i]);
				if ( l1 != 5 ) {
					//System.out.println("la lunghezza non e'=5");
					l2 = Array.getLength(entry.obj_serial[i]);
					//System.out.println("lunghezza del valore del campo=l2="+l2);
					if (l1 != l2) {
						//System.out.println("le lunghezze sono diverse e quindi ciao: ritorno");
						return false ;	
					}
					else { 	
						for(int j = 0; j < l1 ; j++)
							if (obj_serial[i][j] != entry.obj_serial[i][j]) {
								return false; 
							}
					}
				}  
				else {
					//System.out.println("la e' 5 e quindi supponiamo che sia null");
				}

			// Anche se sono diverse puo' aversi che la prima
			// forma serializzata corrisponda a un campo null 
			// allora il match deve avvenire.
		
			// per come e' realizzato il processo di serializzazione
			// un campo null e' lungo 5.
	      
			} 	// fine for
		
	      return true;
		}		// fine if (trovato)
	    
		else {
			//System.out.println("non trovato");
			return false;
		}
	 
	}

/*******************************
 * da qui in poi  la parte meta=true del metodo matches
 ******************************************/ 

	else { // meta  true
		
		boolean trovato = false;
	  	Object objj1,objj2;
	  	int h = 0;
		while( h < Array.getLength(entry.re_cl_serial)) {
			if(re_cl_serial[0] == entry.re_cl_serial[h++])
				trovato = true; 
					//Class cs = SCS.getClass(entry.c1_serial[h]);
					//System.out.println( cs.toString() );
	      }

		if (trovato) {
			int l1,l2;
			int length=Array.getLength(re_obj_serial);
			int length2 = Array.getLength(entry.re_obj_serial);

					//a cosa serve questo???? perche' 4: in realta' gli elementi della ReactionEntry sono 5
			if ((length != 5) && (length2 != 5)) 
			{
				//System.out.println("false");
				return false;
			}
			
			/*
			for(int i = 0; i < length ; i++) {
				System.out.println("Il numero di campi dell'oggetto e':"+
				(Array.getLength(re_obj_serial)));
				System.out.println(re_obj_serial[i].getClass());
				System.out.println(re_obj_serial[i].toString());
			}
			*/

				//qui inizia il controllo sul primo campo dopo la entry

			if (Array.getLength(re_obj_serial[0])!= 5){
				if (Array.getLength(re_obj_serial[0]) != Array.getLength(entry.re_obj_serial[0])) {
					//System.out.println("falso1");
					return false;
				}

				for(int j = 0; j < Array.getLength(re_obj_serial[0]) ; j++) 
				{
		       		if (re_obj_serial[0][j] != entry.re_obj_serial[0][j]){
						//System.out.println("false 2");
						return false; 
					}
					//else System.out.println(j+"  "+obj_serial[0][j]+" "+entry.obj_serial[0][j]);
				}
				//System.out.println("esco dal for su obj_serial[0] che vale "+obj_serial[0].toString());
			}

				//controllo sul secondo campo dopo la entry

			if (Array.getLength(re_obj_serial[1])!= 5) {
				if (Array.getLength(re_obj_serial[1]) != Array.getLength(entry.re_obj_serial[1])) {
					//System.out.println("falso3");
					return false;
				}
				for(int j = 0; j < Array.getLength(re_obj_serial[1]); j++)
					if (re_obj_serial[1][j] != entry.re_obj_serial[1][j])  {
						//System.out.println("falso4");
						return false; 
					}
			}

			//quello che segue e' il controllo sull'ultimo campo, ovvero sul thread reattivo
			//secondo me e' un controllo inutile, anzi, contrario alle specifiche di MARS
			
			/*
				//controllo sul terzo campo:
			if (Array.getLength(entry.re_obj_serial[2])!= 5) {
				if (Array.getLength(re_obj_serial[2]) != Array.getLength(entry.re_obj_serial[2])) {
					//System.out.println("falso5");
					return false;
				}
				for(int j = 0; j < Array.getLength(entry.re_obj_serial[2]); j++)
					if (re_obj_serial[2][j] != entry.re_obj_serial[2][j]) {
						//System.out.println("falso6");
						return false; 
					}
			}
			*/

				//bisogna aggiungere il controllo che appartengano alla stessa classe
			h=0;
			while( h < Array.getLength(cl_serial)) {
				if(entry.cl_serial[0] == cl_serial[h++])
					break;
			}

				//vero e proprio controllo sulla Entry passiva !!!!

			int ll1 =0;
			int ll2 =0;
			int lengtha=Array.getLength(obj_serial);

			for(int i = 0; i < lengtha ; i++) {
				
				//System.out.println(obj_serial[i].getClass());
				//System.out.println(obj_serial[i].toString());
				l1 = Array.getLength(entry.obj_serial[i]);
				l2 = Array.getLength(obj_serial[i]);
				if ( l2 != 5 ) {
	
					if (l1 != l2) return false ;	
						else { 	
							for(int j = 0; j < l1 ; j++)
							if (obj_serial[i][j] != entry.obj_serial[i][j]) 
							return false; 
						}
				}  
			} // fine for

		      return true;
		
		}// fine if (trovato)
		
		else 
			return false;
	}// fine else (meta)
}

/*********************************************************/

public boolean matches(EntryRep entry) 
{
	boolean rit = matches(entry, false);
	return rit;
}


	
/*************************************************************************
 *  Questo  metodo ritorna true se l'EntryRep corrente viene unlocked, 
 *  false in caso contrario.
 *************************************************************************/

  public synchronized boolean unLock(Principal who) 
{
	// se non e' locked, non posso sloccarla
	if (!lock) return false;
	else {
		if ( who == lock_id ) {
			// se chi sta' "sloccando" e' lo stesso che ha loccato
			// la tupla, la puo' "sloccare".
			lock = false;
			return true;
		} 
		else return false;
	}
  }    

/***********************************************************/

public static void SetSpaceClassServer(SpaceClassServer scs) {
	// consento di settare il class server solo una volta
	if(SCS == null) SCS = scs;
  } 

/***********************************************************/

public void setAcl(Acl a) {
	if ( acl == null ) acl = (AclImpl) a;
  } 

/***********************************************************/

/*public Date getDate() {
		return date;
	}*/
 
/***********************************************************/

/*public int compareTo(Date data1, Date data2) {
	long firstTime = data1.getTime();
	long secondTime = data2.getTime();
	return (firstTime < secondTime ? -1: (firstTime == secondTime ? 0:1));
	}

public int compareTo (Object re ) {
	EntryRep rer = (EntryRep)re;
	return (compareTo((Date)this.getDate(), (Date)rer.getDate()));
	}
*/

/***********************************************************/

public int id() {
	int id = 0;
	return id;
}

/***********************************************************/

public void id(long newID) {
id = newID;
}

/***********************************************************/

public void setExpiration(long newExpiration) {
expires = newExpiration;
}

/***********************************************************/

public long getExpiration() {
return expires;
}

/***********************************************************/

public void showValues() {

}

/***********************************************************/

public byte[][] Serializza (Entry e, ClassLoader RCL) {
	byte[][] e_serial = null;
	ByteArrayOutputStream out = new ByteArrayOutputStream(1000);
		
		
		// Faccio uso di un metodo della classe  Class per estrarre
		// i campi come nome e tipo. Una volta individuati i campi
		// e' semplice recuperare gli oggetti corrispondenti e poi 
		// serializzarli. Tale metodo restituisce un array di oggetti
		// istanze della classe Fields (tale classe e' nel package 
		// reflect).
		try {
			DataOutputStream data = new DataOutputStream(out);
			ObjectOutputStream output = new ObjectOutputStream(out);
	
			Class rif = ((Object)e).getClass();
			Field attrib[] = rif.getFields();
			Class c = Class.forName("java.lang.Object");
			Vector SUIDs = new Vector();
			
		// nella entry memorizzo solo il SUID della classe e delle
		// sue superclassi, che sono necessari nel matching.
		// N.B. Prima di serializzare gli oggetti memorizzo anche la 
		// classe della tupla in forma serializzata nello space class
		// server, per potere in  seguito ripristinare l'oggetto.
		
			while(!((Object)c).equals(rif)) {
				
				ObjectStreamClass OSC = 
					ObjectStreamClass.lookup(rif);
				// estraggo il SUID ...
				Long SUID = new Long(OSC.getSerialVersionUID());
				SUIDs.addElement(SUID);
		
				ByteArrayOutputStream out_cl = 
				  new ByteArrayOutputStream(500);
				ObjectOutputStream output_cl = 
				  new ObjectOutputStream(out_cl);
				output_cl.writeObject((Object)rif);
				String tmp = e.getClass().toString();
				SCS.addClass(SUID.longValue(), 
				  tmp.substring(tmp.lastIndexOf(" ")+1),
					     out_cl.toByteArray());
 				System.out.println("###  "+
				      tmp.substring(tmp.lastIndexOf(" ")+1)+
				     "salvata nello SpaceClassServer");
				rif = rif.getSuperclass();
		
}
		// trasformo il vettore SUIDs in un array
		long[] l_serial = new long[SUIDs.size()];
		Enumeration E = SUIDs.elements();
		for( int i = 0; E.hasMoreElements(); i++) 
			l_serial[i] = ((Long)E.nextElement()).longValue();
		
		// ora serializzo i campi ...
		output = new ObjectOutputStream(out);
		output.flush();
		out.reset();
		
		e_serial = new byte[Array.getLength(attrib)][];
		for(int i = 0; i < Array.getLength(attrib); i++) {
			Object obj = attrib[i].get(e);
			output = new ObjectOutputStream(out);
			output.writeObject(obj);
			e_serial[i] = out.toByteArray();
			output.flush();
			out.reset();
		}

	     

  	  	} catch (IllegalAccessException ex)
   	   	{System.err.println("The object is inaccessible(1):"+ex);}
	  	 catch (ClassNotFoundException ex)
		{System.out.println("Class not found section number (1):"+ex);} 		

catch (IOException ex)
	  	{System.err.println("IOException in section number (1):"+ex);}
	
 return e_serial;
}}
