package jive.javaSpace;

/*************************************************************************
 *  Questa classe ha la funzione di interfaccia alle primitive di accesso
 *  allo spazio messe a disposizione dallo spacekernel
 *  Inoltre le considerazioni corrispondono al caso in cui la operazione 
 *  non e' bloccante, se invece lo e' l'operazione ritornera' comunque 
 *  almeno una tupla matchante, eventualmente bloccandosi in attesa che ne
 *  sia inserita una nello spazio.  
 *************************************************************************/

import java.util.*;
import jive.util.*;
import java.security.Principal;
import jive.security.*;
import jive.util.Passer;
import net.jini.core.event.*;
import net.jini.core.lease.*;
import java.security.acl.*;
import java.rmi.*;
import java.lang.reflect.*;

public class MetaSpace  extends PassiveSpace 
{

	private Passer pout = null;
 
	SpaceVector blackboard = new SpaceVector();

		// identita' dell'agente che possiede l'interfaccia
	private Principal myId;

	protected LeaseMarsPolicy metaMars = new LeaseMarsPolicy (200000, 100000);    


public MetaSpace (Principal identity,Passer Pout  ) 
{
	super(identity, Pout);
	pout = Pout; 	
	myId = identity;
}


/*************************************************************************
 *  Questo metodo esegue la read, se non vi sono elementi nello spazio 
 *  che matchano con il template ritorna un array vuoto. In particolare 
 *  questo metodo e' quello che deve essere invocato dalla reazione.
 *************************************************************************/  

public Entry read (Entry entry, Transaction txn,long timeout, Principal who) 
	throws NotInterfaceOwnerException, NoPermissionException 
{
	/*
	if ( !super.checkPermission(who,new PermissionImpl("META")) )
		throw new NoPermissionException();
	if ( !super.checkPermission(who,new PermissionImpl("READ")) )
		throw new NoPermissionException();
	*/
	 
	EntryRep rep = new EntryRep(entry,true);
	//System.out.println("prima della rd meta"+timeout);
	EntryRep resultrep = this.rd(rep, txn,timeout, who);
	//System.out.println("il ritorno di rd "+ resultrep);
	Entry result = null;
	
	if (resultrep != null) { 
		result = resultrep.entry(true);
		// "slocco" la tupla
		resultrep.unLock(who);
	}
	
	this.notifing();
	return result;
}  


/*************************************************************************
 *  Questo metodo esegue la read e restituisce tutte le tuple che matchano con 
 *  il template.  Si differenzia dall'altro metodo di read siccome questo non e' 
 *  reattivo e verra' utilizzato nella reazione.
 *************************************************************************/ 

public Entry[] readAll (Entry entry, Transaction txn,long timeout, Principal who)
	throws NotInterfaceOwnerException, NoPermissionException 
{
	/*
	if ( !checkPermission(who,new PermissionImpl("META")) )
		throw new NoPermissionException();
	if ( !super.checkPermission(who,new PermissionImpl("READ")) )
		throw new NoPermissionException();
	*/

	if (entry == null) {

		Enumeration enum = blackboard.elements();
		Entry[] ritorna = new Entry[blackboard.size()];
		int i = 0;
		for(;enum.hasMoreElements();) {
			EntryRep list = (EntryRep)enum.nextElement();
			ritorna[i] = list.entry(true);
			i++;
		}
		return ritorna;
	}

	EntryRep rep = new EntryRep(entry,true);	 
		// segnalo allo spacevector che la operazione e' in corso
	blackboard.multipleOp(true);
	EntryRep[] arrayrep = rdAll(rep,txn,timeout,who);
	Entry[] array = new Entry[arrayrep.length];
	for (int i = 0; i < arrayrep.length ; i++) {
		array[i] = arrayrep[i].entry(true);
		arrayrep[i].unLock(who);
	}
		 // segnalo allo spacevector che la operazione e' finita
	blackboard.multipleOp(false);
		// dico allo spacekernel di riorganizzare il vettore
	blackboard.reOrganize();
		// risveglio gli agenti in attesa sulle tuple lockate
	this.notifing();
	return array;
  } 

 
/*************************************************************************
 *  Questo metodo esegue la take, se non vi sono elementi nello spazio 
 *  matchanti ritorna null. Se elementi matchano ne cancella uno e lo 
 *  ritorna. Si differenzia dall'altro metodo di take siccome questo non 
 *  e' reattivo e verra' utilizzato dalla reazione.
 *************************************************************************/ 

public Entry take (Entry entry, Transaction txn,long timeout, Principal who) 
	throws NotInterfaceOwnerException, NoPermissionException 
{
	/*
	if ( !super.checkPermission(who,new PermissionImpl("META")) )
		throw new NoPermissionException();
	if ( !super.checkPermission(who,new PermissionImpl("TAKE")) )
		throw new NoPermissionException();
	*/	

	EntryRep rep = new EntryRep(entry,true);
	EntryRep resultrep = this.in( rep, txn,timeout, who);
	Entry result = null;
	if (resultrep != null) result = resultrep.entry(true);
	this.notifing();
	return result; 
}

  
/*************************************************************************
 *  Questo metodo esegue la take, se non vi sono elementi nello spazio 
 *  che fanno match viene ritornato un array con nessun elemento. In 
 *  particolare questo metodo e' quello che deve essere invocato dal thread
 *  che implementa la reattivita', inquanto qui non e' prevista la reazione.
 *************************************************************************/

public Entry[] takeAll (Entry entry, Transaction txn,long timeout, Principal who)
	throws NotInterfaceOwnerException, NoPermissionException 
{
	/*
	if ( !super.checkPermission(who,new PermissionImpl("META")) )
		throw new NoPermissionException();
	if ( !super.checkPermission(who,new PermissionImpl("TAKE")) )
		throw new NoPermissionException();
	*/

	EntryRep rep = new EntryRep(entry,true);
		// segnalo allo spacevector che e' in corso una operazione
		// multipla.
	blackboard.multipleOp(true);
	 
	EntryRep[] arrayrep = this.inAll(rep,txn,timeout,who);

	Entry[] array = new Entry[arrayrep.length];
	for (int i = 0; i < arrayrep.length; i++)
		array[i] = arrayrep[i].entry(true); 

	 		// segnalo allo spacevector che la operazione e' finita
	blackboard.multipleOp(false);
	 
		// dico allo spacekernel di riorganizzare il vettore
	blackboard.reOrganize();
	 
	 // non e' necessario eseguire l'unLock delle tuple siccome 
	 // sono state eliminate dallo spazio, quindi non e' nemmeno 
	 // necessario risvegliare gli agenti in attesa sulle tuple lockate
	
	return array;
}
  
/*************************************************************************
 *  Questo metodo memorizza nello spazio una entry, ma a differenza del 
 *  precedente, questo non prevede lo scatenarsi della reazione.
 *************************************************************************/ 

public Lease write (Entry entry, Transaction txn,long lease, Principal who) 
	throws NotInterfaceOwnerException, NoPermissionException 
{
	/*
	if ( !super.checkPermission(who,new PermissionImpl("META")) )
		throw new NoPermissionException();
	//System.out.println("paganini");
	if ( !super.checkPermission(who,new PermissionImpl("WRITE")) )
		throw new NoPermissionException();
	*/

	if (entry == null)
		System.out.println("no arg valido");
	//throws NullPointerException;
	EntryRep rep = new EntryRep(entry, true);
	 
		// inserisco la tupla nello spazio
	Lease leasemeta = out(rep, txn,lease, who);

		//if (leasemeta != null)
	 
	//System.out.println("dopo out");
		
		// "slocco" la tupla
	if (rep != null) rep.unLock(who);
	 
		// risveglio gli agenti in attesa sulle tuple lockate
	this.notifing();

	return leasemeta;
	
} 


/*************************************************************************
 * Questo metodo riceve un indice (index), un template (rep), una identita'
 * (who), e una istanza di transaction (txn). Quello che fa' e' di 
 * verificare se la tupla i-esima dello spazio (se c'e') combina con il 
 * template passato, se possiede i diritti per leggere la tupla e se non e'
 * locked.
 * Nel caso in cui la verifica vada a buon fine la tupla viene "loccata"
 * e restituita. 
 *************************************************************************/
  
private synchronized EntryRep get(int index, EntryRep rep, 
	Principal who, PermissionImpl perm) 
{
	EntryRep E = null;
	try {
		if( (E = (EntryRep)blackboard.elementAt(index)) != null ) {
			//System.out.println("ho trovato sulla metablackboard: "+E);
			if ( rep.matches(E,true) && (!E.isLocked(who)) && 
				( (E.getAcl() == null) ||(isOwner(who)) ||
				E.getAcl().checkPermission(who, perm) ))
			{	
					// se la tupla soddisfa tutti i requisiti la "locco"
				E.lock(who);
			}
					// altrimenti setto a null E
			else {
				E = null;
			}
		}
	} 
	catch(ArrayIndexOutOfBoundsException e){
			// se l'indice e' riferito ad una tupla che e' gia' stata 
			// eliminata, setto E (che poi verra' ritornato) a null
		E = null;
	}
	   
	notifyAll();
	return E;
}
  

/*************************************************************************
 * Questo metodo effettua la classica out di Linda, cioe' la write,
 * implementando anche la gestione dei diritti.
 *************************************************************************/

synchronized Lease out(EntryRep rep, Transaction txn,long lease, Principal who) 
{
	Lease metaLease = null;

	try {
		if (lease != 0)
			metaLease = metaMars.leaseFor(rep, lease);
	} 
	catch (LeaseDeniedException e) {
		System.out.println("eccezione in lease"); }
	//throw new InternalSpaceException ("Spazio di base: entryMars." +
	//"leaseFor threw LeaseDeniedException", e);
	blackboard.addElement(rep);
	
	// locco la tupla per imporre la sua presenza al
		// momento dello scatenarsi delle reazioni
	rep.lock(who);		

	return metaLease;
}  

/*************************************************************************
 * Questo metodo effettua la classica read di Linda, implementando anche 
 * il locking delle tuple e la gestione dei diritti.
 *************************************************************************/
  
synchronized EntryRep rd (EntryRep rep, Transaction txn,long timeout,
		Principal who ) 
{
	/*	
	Printer.print_out(pout, "SK: I'm looking for the matching"+
		" entry to read...");
	*/

	EntryRep ER = null;
	boolean found = false;

	while (!found) {
		for (int i=0; i<size() && !found ; i++) {
			ER = get(i,rep,who,new PermissionImpl("READ"));
			if (ER != null) {
				found = true;
			}
		}
		if (!found && txn.block()) 
			waiting();
		else 
			break;
	}
	
	return ER;	
}


/*************************************************************************
 * Questo metodo effettua la readAll di Linda, implementando anche 
 * il locking delle tuple e la gestione dei diritti.
 *************************************************************************/
  
synchronized EntryRep[] rdAll (EntryRep rep, Transaction txn,long timeout,
		Principal who ) 
{
	/*
	Printer.print_out(pout, "SK: I'm looking for the matching"+
		      " entry to read...");
	*/
	
	boolean found = false;
	Vector v = new Vector();

	while (!found) {
		for (int i=0; i<size() ; i++) {
			EntryRep ER = get(i,rep,who,new PermissionImpl("READ"));
			if (ER != null) {
				v.addElement(ER);
				found = true;
			}					
		}
		if (!found && txn.block()) 
			waiting();
		else 
			break;
	}		
	
	EntryRep[] array = new EntryRep[v.size()]; 
	Enumeration e = v.elements();
	for(int i=0;e.hasMoreElements();i++)
		array[i] = (EntryRep)e.nextElement();
		
	return array;	
  }  



/*************************************************************************
 * Questo metodo effettua la classica in di Linda, cioe' la take, 
 * implementando anche il locking delle tuple e la gestione dei diritti.
 *************************************************************************/

synchronized EntryRep in (EntryRep rep, Transaction txn,long timeout,
		Principal who ) 
{
	
	/*
	Printer.print_out(pout, "SK: I'm looking for the matching"+
		" entry to take...");
	*/

	EntryRep ER = null;
	boolean found = false;
	
	while (!found) {
		for (int i=0; i<size() && !found ; i++) {
			ER = get(i,rep,who,new PermissionImpl("TAKE"));
			if (ER != null) {
				blackboard.removeElementAt(i);
				found = true;
			}					
		}
	   	if (!found && txn.block()) 
			waiting();
		else 
			break;
	}		
	
	return ER;	
}


/*************************************************************************
 * Questo metodo effettua la classica inAll di Linda, cioe' la takeAll, 
 * implementando anche il locking delle tuple e la gestione dei diritti.
 *************************************************************************/
  
synchronized EntryRep[] inAll (EntryRep rep, Transaction txn,long timeout,
		Principal who ) 
{
	/*
	Printer.print_out(pout, "SK: I'm looking for the matching"+
		      " entry to take...");
	*/

	boolean found = false;
	Vector v = new Vector();
	
	while (!found) {
		for (int i=0; i<size() ; i++) {
			EntryRep ER = get(i,rep,who,new PermissionImpl("TAKE"));
			if (ER != null) {
				v.addElement(ER);
				blackboard.removeElementAt(i);
				found = true;
			}					
		}
		if (!found && txn.block()) 
			waiting();
		else 
			break;
	}		
	
	EntryRep[] array = new EntryRep[v.size()]; 
	Enumeration e = v.elements();
	for(int i=0;e.hasMoreElements();i++)
		array[i] = (EntryRep)e.nextElement();
		
	return array;	
  }  


/*************************************************************************
 * Questo metodo fornisce il numero di tuple nello space. 
 *************************************************************************/

public synchronized int size() 
{
	return blackboard.size();
} 

}