package jive.javaSpace;

/*************************************************************************
 *  Questa classe ha la funzione di interfaccia alle primitive di accesso
 *  allo spazio messe a disposizione dallo spacekernel
 *  Inoltre le considerazioni corrispondono al caso in cui la operazione 
 *  non e' bloccante, se invece lo e' l'operazione ritornera' comunque 
 *  almeno una tupla matchante, eventualmente bloccandosi in attesa che ne
 *  sia inserita una nello spazio.  
 *************************************************************************/

import java.util.*;
import jive.util.*;
import java.security.Principal;
import jive.security.*;
import java.security.acl.*;
import java.security.*;
import jive.util.Passer;
import net.jini.core.event.*;
import net.jini.core.lease.*;
import java.rmi.*;
import java.lang.reflect.*;


public class PassiveSpace  extends AclImpl implements MARSInt {

	private Passer pout = null;
	
	SpaceVector blackboard = new SpaceVector();
  	
		// identita' dell'agente che possiede l'interfaccia
	private Principal myId;

	protected LeaseMarsPolicy entryMars = new LeaseMarsPolicy (20000000, 100000);   

		// manager delle notifiche
	Notifier nManager;
	NotifyRecord notifyrecord;
	
		// space class server  
	private SpaceClassServer SCServer;
	int FOREVER;

/**************************************************************
 * costruttori
 ***********************************************************/

public PassiveSpace (Principal creator) {
	super( creator);
}

public PassiveSpace (Principal creator,Passer Pout ) {
	super( creator);
	pout = Pout; 	
	myId = creator;
	
		// creo uno space Class server con il compito di 
		// memorizzare le classi delle entry.
	EntryRep.SetSpaceClassServer(new SpaceClassServer());
	NotifyRecord notifyrecord = new NotifyRecord(pout);

	//Notifier nManager = new Notifier(ppout);
}

/**************************************************************
 * metodi
 ***********************************************************/

/*
public void print(String msg) 
{
	Date now = new Date();
	String prefix ="("+now.getHours()+":"+now.getMinutes()+":"+
	    now.getSeconds()+") "+myId.getName()+": ";

	if (pout == null)
		System.out.println(prefix+msg);
	else
		pout.put(prefix+msg);
}
*/


public Passer getPasser() 
{
	return pout;
}



/*************************************************************************
 *  Questo metodo esegue la read, se non vi sono elementi nello spazio 
 *  che matchano con il template ritorna un array vuoto. In particolare 
 *  questo metodo e' quello che deve essere invocato dalla reazione.
 *************************************************************************/  

public Entry read (Entry entry, Transaction txn,long timeout, Principal who) 
	  throws NotInterfaceOwnerException, NoPermissionException 
{
	
	if ( !checkPermission(who,new PermissionImpl("READ")) )
		throw new NoPermissionException();
	if ( !checkPermission(who,new PermissionImpl("PASSIVE")) )
		throw new NoPermissionException();
	

 	EntryRep  rep = new EntryRep(entry);
 	
		//invochiamo la tipica entry di Linda; ci restituisce una EntryRep
	EntryRep resultrep = rd(rep, txn,timeout, who);
	
		//dobbiamo convertire il risultato in una Entry
	Entry result = null;

	if (resultrep != null) { 
		result = resultrep.entry(false);

	    		// "slocco" la tupla
	    	resultrep.unLock(who);
	 }
	 
	 notifing();

	/*
	System.out.println("sono la read e sto per ritornare la seguente tupla");
	result.showValues();
	*/

	 return result;
}  


/*************************************************************************
 *  Questo metodo esegue la readAll e restituisce tutte le tuple che matchano con 
 *  il template.  Si differenzia dall'altro metodo di read siccome questo non e' 
 *  reattivo e verra' utilizzato nella reazione.
 *************************************************************************/ 

public Entry[] readAll(Entry entry, Transaction txn,long timeout, Principal who)
	throws NotInterfaceOwnerException, NoPermissionException 
{

	
	if ( !checkPermission(who,new PermissionImpl("PASSIVE")) )
		throw new NoPermissionException();
	if ( !checkPermission(who,new PermissionImpl("READ")) )
		throw new NoPermissionException();
	


		//se invochiamo la readAll senza fornire nessuna entry, significa che
		//vogliamo leggere tutte le tuple presenti nello spazio
	if (entry == null) {
		Enumeration enum = blackboard.elements();
		Entry[] ritorna = new Entry[blackboard.size()];
		int i = 0;
		for(;enum.hasMoreElements();) {
			EntryRep list = (EntryRep)enum.nextElement();
			ritorna[i] = list.entry(false);
			i++;
		}
	
		return ritorna;
	}


	else {
		EntryRep rep = new EntryRep(entry);	 
			// segnalo allo spacevector che la operazione e' in corso
		blackboard.multipleOp(true);
	 
		EntryRep[] arrayrep = this.rdAll(rep,txn,timeout,who);
		Entry[] array = new Entry[arrayrep.length];
	 
		for (int i = 0; i < arrayrep.length ; i++) {
			array[i] = arrayrep[i].entry();
			arrayrep[i].unLock(who);
		}

	 		// segnalo allo spacevector che la operazione e' finita
		blackboard.multipleOp(false);
	 
			// dico allo spacekernel di riorganizzare il vettore
		blackboard.reOrganize();
	 
			// risveglio gli agenti in attesa sulle tuple lockate
		this.notifing();	
		return array;
	} 
}  


/*************************************************************************
 *  Questo metodo esegue la take, se non vi sono elementi nello spazio 
 *  matchanti ritorna null. Se elementi matchano ne cancella uno e lo 
 *  ritorna. Si differenzia dall'altro metodo di take siccome questo non 
 *  e' reattivo e verra' utilizzato dalla reazione.
 *************************************************************************/ 

public Entry take(Entry entry, Transaction txn,long timeout, Principal who) 
	throws NotInterfaceOwnerException, NoPermissionException 
{
	
	if ( !checkPermission(who,new PermissionImpl("PASSIVE")) )
		throw new NoPermissionException();
	if ( !checkPermission(who,new PermissionImpl("TAKE")) )
		throw new NoPermissionException();
		

	EntryRep rep = new EntryRep(entry);
	
		//eseguo la classica in di Linda
		//mi viene restituita una EntryRep che devo convertire in Entry
	EntryRep resultrep = this.in( rep, txn,timeout,who);
	
	Entry result = null;
	if (resultrep != null) {
		result = resultrep.entry();
	}

	//notifing();
	return result; 
}  


/*************************************************************************
 *  Questo metodo esegue la take, se non vi sono elementi nello spazio 
 *  che fanno match viene ritornato un array con nessun elemento. In 
 *  particolare questo metodo e' quello che deve essere invocato dal thread
 *  che implementa la reattivita', inquanto qui non e' prevista la reazione.
 *************************************************************************/

public Entry[] takeAll(Entry entry, Transaction txn,long timeout, Principal who)
	throws NotInterfaceOwnerException, NoPermissionException 
{
	 
	if ( !checkPermission(who,new PermissionImpl("PASSIVE")) )
		throw new NoPermissionException();

	if ( !checkPermission(who,new PermissionImpl("TAKE")) )
		throw new NoPermissionException();
		

	EntryRep rep = new EntryRep(entry);
	
		// segnalo allo spacevector che e' in corso una operazione
		// multipla.
	blackboard.multipleOp(true);
	
		//eseguo la classica inALL di Linda.
		//mi verra' restituito un array di EntryRep che
		//dovro' convertire in Entry 
	EntryRep[] arrayrep = this.inAll(rep,txn,timeout,who);
	
	Entry[] array = new Entry[arrayrep.length];
	for (int i = 0; i < arrayrep.length; i++)
	      array[i] = arrayrep[i].entry(); 
	
		 // segnalo allo spacevector che la operazione e' finita
	blackboard.multipleOp(false);
	 
	 	// dico allo spacekernel di riorganizzare il vettore
	blackboard.reOrganize();
	 
		// non e' necessario eseguire l'unLock delle tuple siccome 
		//sono state  dallo spazio, quindi non e' nemmeno 
		// necessario risvegliare gli agenti in attesa sulle tuple lockate
	
	return array;
}

  

/*************************************************************************
 *  Questo metodo memorizza nello spazio una entry, ma a differenza del 
 *  precedente, questo non prevede lo scatenarsi della reazione.
 *************************************************************************/ 
  
public Lease write 
	(Entry entry, Transaction txn,long lease, Principal who) 
	throws NotInterfaceOwnerException, NoPermissionException 
{
	
	if ( !checkPermission(who,new PermissionImpl("PASSIVE")) )	
		throw new NoPermissionException();

	if ( !checkPermission(who,new PermissionImpl("WRITE")) )
		throw new NoPermissionException();
	
	if (entry == null) {
		System.out.println("no argomento valido");
		return null;
	}
	//throws NullPointerException;
	
	else {
		EntryRep rep = new EntryRep(entry, false);

			// inserisco la tupla nello spazio
		Lease leasentry = out(rep, txn,lease, who);
	
		//if (leasentry != null) 
		long exp = rep.getExpiration();
		final long now = System.currentTimeMillis();
		long duration = exp - now;
  
			// "slocco" la tupla
		if (rep != null) 
			rep.unLock(who);
	 
			// risveglio gli agenti in attesa sulle tuple lockate
		notifing();

		/*
			Vector chits;
			cerco se vi sono registrazioni di evento da inviare.
			chits = notifyrecord.findNotify( rep, this);
			if (chits.size() != 0) {
			Notifier.doNotify(chits); 
			for (int i=0; i<chits.size(); i++) {
			System.out.println(" la registrazione "+ chits.elementAt(i));
			}
			} 
		*/

		return leasentry;
	}
 } 


/*************************************************************************
 * Questo metodo riceve un indice (index), un template (rep), una identita'
 * (who), e una istanza di transaction (txn). Quello che fa' e' di 
 * verificare se la tupla i-esima dello spazio (se c'e') combina con il 
 * template passato, se possiede i diritti per leggere la tupla e se non e'
 * locked.
 * Nel caso in cui la verifica vada a buon fine la tupla viene "loccata"
 * e restituita. 
 *************************************************************************/

private synchronized EntryRep get(int index, EntryRep rep, 
	Principal who, PermissionImpl perm) 
{
	EntryRep E = null;
	try {
		if( (E = (EntryRep)blackboard.elementAt(index)) != null ) {
		
			if ( rep.matches(E,false) && (!E.isLocked(who)) && 
				( (E.getAcl() == null) ||(isOwner(who)) ||
				E.getAcl().checkPermission(who, perm) ))	
			{
	 				// se la tupla soddisfa tutti i requisiti la "locco"
				E.lock(who);
			}
				// altrimenti setto a null E
			else {
				E = null;
				
		     }
		}
	} 

	catch(ArrayIndexOutOfBoundsException e)
	{	
		// se l'indice e' riferito ad una tupla che e' gia' stata 
		// eliminata, setto E (che poi verra' ritornato) a null
		E = null;
	}
	   
	notifyAll();
	return E;
}  


/*************************************************************************
 * Questo metodo effettua la classica in di Linda, cioe' la take, 
 * implementando anche il locking delle tuple e la gestione dei diritti.
 *************************************************************************/

synchronized EntryRep in (EntryRep rep, Transaction txn,long timeout,
		Principal who ) 
{

	/*
	Printer.print_out(pout, "SK: I'm looking for the matching"+
		" entry to take...");
	*/

	EntryRep ER = null;
	boolean found = false;
	while (!found) {
		for (int i=0; i<size() && !found ; i++) {
			ER = get(i,rep,who,new PermissionImpl("TAKE"));
			if (ER != null) {
				blackboard.removeElementAt(i);
				found = true;
			}					
		}
	   	if (!found && txn.block()) 
			waiting();
		else 
			break;
	}		
	
	return ER;	
}

/*************************************************************************
 * Questo metodo effettua la classica inAll di Linda, cioe' la takeAll, 
 * implementando anche il locking delle tuple e la gestione dei diritti.
 *************************************************************************/
  
synchronized EntryRep[] inAll (EntryRep rep, Transaction txn,long timeout,
		Principal who ) {
	
	/*
	Printer.print_out(pout, "SK: I'm looking for the matching"+
		      " entry to take...");
	*/

	boolean found = false;
	Vector v = new Vector();
	
	while (!found) {
		for (int i=0; i<size() ; i++) {
			EntryRep ER = get(i,rep,who,new PermissionImpl("TAKE"));
			if (ER != null) {
				v.addElement(ER);
				blackboard.removeElementAt(i);
				found = true;
			}					
		}
		if (!found && txn.block()) 
			waiting();
		else 
			break;
	}		
	
	EntryRep[] array = new EntryRep[v.size()]; 
	Enumeration e = v.elements();
	for(int i=0;e.hasMoreElements();i++)
		array[i] = (EntryRep)e.nextElement();
		
	return array;	
  }  


/*************************************************************************
 * Questo metodo invoca la notifyAll sulla risorsa sincronizzata e viene 
 * utilizzata dai metodi di take e read all, passive e non.
 *************************************************************************/

  synchronized void notifing() 
{
	notifyAll();
}  


/*************************************************************************
 * Questo metodo effettua la classica out di Linda, cioe' la write,
 * implementando anche la gestione dei diritti.
 *************************************************************************/
	
synchronized Lease out(EntryRep rep, Transaction txn,long lease, Principal who) 
{
	Lease entryLease = null;
	//Printer.print_out(pout, "SK: I'm writing in the Space!"); 

	try {
		if (lease != 0)
			entryLease = entryMars.leaseFor(rep, lease);
	} 
	catch (LeaseDeniedException e) {
		System.out.println("eccezione in lease"); 
	}
	
	//throw new InternalSpaceException ("Spazio di base: entryMars." +
	//"leaseFor threw LeaseDeniedException", e);

	blackboard.addElement(rep);
		// locco la tupla per imporre la sua presenza al
		// momento dello scatenarsi delle reazioni
	rep.lock(who);		
	
	return entryLease;
}  

 
/*************************************************************************
 * Questo metodo effettua la classica read di Linda, implementando anche 
 * il locking delle tuple e la gestione dei diritti.
 *************************************************************************/
  
synchronized EntryRep rd (EntryRep rep, Transaction txn,long timeout,
		Principal who ) 
{
	//Printer.print_out(pout, "SK: I'm looking for the matching"+
		//" entry to read...");
	EntryRep ER = null;
	boolean found = false;
	while (!found) {
		for (int i=0; i<size() && !found ; i++) {
			ER = get(i,rep,who,new PermissionImpl("READ"));
			if (ER != null) {
				found = true;
			}
		}
		if (!found && txn.block()) 
			waiting();
		else 
			break;
	}
	
	return ER;	
}

/*************************************************************************
 * Questo metodo effettua la readAll di Linda, implementando anche 
 * il locking delle tuple e la gestione dei diritti.
 *************************************************************************/
  
synchronized EntryRep[] rdAll (EntryRep rep, Transaction txn,long timeout,
		Principal who ) 
{
	//Printer.print_out(pout, "SK: I'm looking for the matching"+
	//	      " entry to read...");
	boolean found = false;
	Vector v = new Vector();
	while (!found) {
		for (int i=0; i<size() ; i++) {
			EntryRep ER = get(i,rep,who,new PermissionImpl("READ"));
			if (ER != null) {
				v.addElement(ER);
				found = true;
			}					
		}

		if (!found && txn.block()) 
			waiting();
		else 
			break;
	}		
	
	EntryRep[] array = new EntryRep[v.size()]; 
	Enumeration e = v.elements();
	for(int i=0;e.hasMoreElements();i++)
		array[i] = (EntryRep)e.nextElement();
		
	return array;	
  }  


/*************************************************************************
 * Questo metodo fornisce il numero di tuple nello space. 
 *************************************************************************/

public synchronized int size() 
{
	return blackboard.size();
}  


/*************************************************************************
 * Questo metodo invoca la wait sulla risorsa sincronizzata e viene 
 * utilizzata dai metodi di take e read all, passive e non.
 *************************************************************************/

synchronized void waiting() {

	try { 
		Printer.print_out(pout, "None entry matches the template, I wait!");
		wait();
	} 
	catch (InterruptedException e)
		{Printer.print_out(pout, "Someone killed me while I was waiting!");}
}
 
/*****************************************************************/
 
public EventRegistration notify (Entry tmpl, Transaction tr, RemoteEventListener
	listener, long leaseTime, Object handback)
	throws  RemoteException, NullPointerException //, TransactionException 
{
	if (listener == null){
		throw new NullPointerException("Listener null" );
	}
	
	EntryRep tmplrep = new EntryRep(tmpl, false);
	NotifyChit ar = notifyrecord.addRecord (tmplrep, tr, listener, leaseTime, handback,this,false); // o Vector
	if (ar == null)
		return null;
	else
		return ar.reg;
}

/*****************************************************************/

public Vector getNotifyVector() 
{
	return notifyrecord.getNotifyVector();
}

/*****************************************************************/

public NotifyRecord getNotifyRecord() 
{
	return notifyrecord;
}

/*****************************************************************/

public Principal getIdentity() 
{
	Principal io = this.myId;
	System.out.println(io);
	return io;
}

/*****************************************************************/    

public synchronized void expire() 
{
	Printer.print_out(this.pout, "--- ExpiredThread is active on the space"+
		this);
	   
	EntryRep E = null;
	for (int i=0;i< size(); i++){
		try {
			if ((E = (EntryRep)blackboard.elementAt(i))!=null ){
				if ((E.getExpiration()<System.currentTimeMillis())){
					Printer.print_out(this.pout, ""+E.getExpiration());
					Printer.print_out(this.pout, ""+System.currentTimeMillis());
					blackboard.removeElementAt(i);
				}
			}
		} 
		catch (ArrayIndexOutOfBoundsException e){ 
			System.out.println("vecchie entry non rimosse");
		}
	}
	Printer.print_out(this.pout, "--- ExpiredThread has token the oldest entries from"+
      		this);

}

} //CHIUSURA DELLA CLASSE 