 package jive.javaSpace;

/*************************************************************************
 *  Questa classe ha la funzione di interfaccia alle primitive di accesso
 *  allo spazio messe a disposizione dallo spacekernel, consentendo anche 
 *  di scatenare reazioni.
 *  I comportamenti descritti in testa ad ogni metodo sono quelli che si 
 *  avrebbero se nessuna reazione attuasse modifiche al comportamento 
 *  della operazione tramite delle reazioni.
 *  Inoltre le considerazioni corrispondono al caso in cui la operazione 
 *  non e' bloccante, se invece lo e' l'operazione ritornera' comunque 
 *  almeno una tupla matchante, eventualmente bloccandosi in attesa che ne
 *  sia inserita una nello spazio.  
 *************************************************************************/

import java.util.*;
import jive.util.*;
import java.security.*;
import jive.security.*;
import jive.util.Passer;
import java.lang.reflect.*;
import net.jini.core.lease.*;

public class ReactiveSpace extends Space 
{
 
		// spazio delle reazioni
	public MetaSpace mspace;
		// gestore delle reazioni
	private RThreadManager TManager;
		// spazio passivo su cui le reazioni agiscono
	public PassiveSpace pspace;
  
	public Passer pout = null;

		// identita' dell'agente che possiede l'interfaccia
	private Principal myId;

	//private AclImpl acl_base=new AclImpl(myId);


/*********************************************************
 *	costruttori
 *********************************************************/

public ReactiveSpace(Principal Identity,Passer Pout, PassiveSpace Ps,
	MetaSpace Rs) 
{
		 //questa si rifa' al costruttore di Space
	super(Identity, Pout); 
		//spazio passivo
	pspace = Ps;
		// lo spazio delle reazioni
	mspace = Rs;
	pout = Pout;
}


public ReactiveSpace(Principal Identity,Passer Pout,PassiveSpace ps, 
	MetaSpace RS, RThreadManager RTM) 
{
		 //questa si rifa' al costruttore di Space
	super(Identity, Pout);
	pspace = ps;
		// lo spazio delle reazioni
	mspace = RS;
	long NO_WAIT = 0;
		// il gestore dei thread reattivi
	TManager = RTM;
		// assegno l'identita' passata al costruttore a myId
	myId = Identity;
}


/*********************************************************
 *	metodi
 *********************************************************/

/*************************************************************************
 *  Questo metodo esegue la read, se vi sono elementi che fanno match 
 *  nello spazio ritorna uno di questi, altrimenti ritorna null.
 *************************************************************************/

public Entry read (Entry entry, Transaction txn, long timeout)
		throws NotInterfaceOwnerException,NoPermissionException 
{
	 	 
		// if (!myId.equals(who)) 
		//   throw new NotInterfaceOwnerException(); 
	long NO_WAIT = 0;
	
		//per il momento lascio queste come commento
	if ( !pspace.checkPermission(myId,new PermissionImpl("REACTIVE")) )
	throw new NoPermissionException();
	if ( !pspace.checkPermission(myId,new PermissionImpl("READ")) )
	throw new NoPermissionException();

		// la parte reattiva si riduce a queste operazioni...
		// individuo tutte le reazioni che devono essere scatenate
		// tupla serializzata

	EntryRep rep = new EntryRep(entry);
	 
		//  tupla "loccata"
	EntryRep lock = null;
	 
		//  qui finira' il risultato del Launch del Thread
	Entry er = null; 
	
	//EntryRep errep; secondo me questa non serve a niente

	EntryRep resultrep = pspace.rd(rep, txn,timeout, myId);
	lock = resultrep;
	Entry result = entry; //"partiamo" dal valore di entry e non da null
		//ovviamente per sfruttarlo nella def di reactionentry

	if (resultrep != null) 
		result = resultrep.entry();

	//System.out.println("oggetto");
	
		// a questo punto result contiene o il template con cui abbiamo
		// fatto la read oppure il risultato della read, se questo non
		//e' null.
		//si rispettano dunque tutte le specifiche di MARS


		/****** PARTE REATTIVA DELLA READ ******/
	 	
	ReactionEntry reactionentry = new ReactionEntry(result,myId,"read",null);  

	Entry[] react;
	Entry tmp = null;
	Entry reactionpassa = null;

		//andiamo a vedere nello spazio di metalivello se ci sono tuple
		//corrispondenti in base alle quali scatenare delle reazioni
		//le mettiamo nell'array react

	/*
	System.out.println("ReactiveSpace: cerco reazioni nel metaspazio");
	System.out.println("uso il template:");
	reactionentry.showValues();
	*/

	react = mspace.readAll(reactionentry,txn,10000,myId);
 		//react = mspace.readAll(null,txn,NO_WAIT,null);

	if (react.length != 0) {
			

			//sono state trovare delle reazioni; per ognuna invochiamo
			//il TManager....


		for (int i=0; i<react.length;i++) {
		
				//ATTENZIONE: PER IL PRIMO PARAMETRO DA PASSARE A LAUNCH BISOGNA SELEZIONARE
				//IL CAMPO reaction DI react[i] e non react[i]

			try {
					//System.out.println("la reattivita' della read ha trovato la tupla");
					//react[i].showValues();
				Class classe= ((Object)react[i]).getClass();
				Field campo=classe.getField("reaction");
					//System.out.println("X"+yy.get(react[i]));

		 		tmp = TManager.Launch((ReactiveThread)campo.get(react[i]),reactionentry, 
				new Space(myId,pout,pspace), myId,null, reactionpassa);
			
				reactionpassa = tmp;	
					//}
					//System.out.println("Step 1");
				er = tmp; 
					//non ho capito perche' tmp viene assegnato sia 
					//a reactionpassa che a tmp

			}
			catch(NoSuchFieldException x) {System.out.println("no such field");}
			catch(IllegalAccessException y) {System.out.println("illegal");}

		}	
	 
			 // libero la tupla locked
		if (lock != null) 
			lock.unLock(myId);	 
	
			// risveglio gli agenti in attesa sulle tuple "loccate"
		pspace.notifing();

	}

	/*
	else 
		System.out.println("la reattivita' della read non ha trovato niente");
	*/

	return result;
 	
}


/**********************************************
 * readAll
 **********************************************/

public Entry[] readAll(Entry entry, Transaction txn, long timeout)
		throws NotInterfaceOwnerException,NoPermissionException 
{
	long NO_WAIT = 0;
	
		//per il momento lascio queste come commento
	if ( !pspace.checkPermission(myId,new PermissionImpl("REACTIVE")) )
	throw new NoPermissionException();
	if ( !pspace.checkPermission(myId,new PermissionImpl("READ")) )
	throw new NoPermissionException();
	
	if (entry == null) {
		Enumeration enum = pspace.blackboard.elements();
		Entry[] ritorna = new Entry[pspace.blackboard.size()];
		int i = 0;
		
		for(;enum.hasMoreElements();) {
			EntryRep list = (EntryRep)enum.nextElement();
			ritorna[i] = list.entry(false);
			i++;
		}
	
		return ritorna;
	}

	else {
		EntryRep rep = new EntryRep(entry);	 

		EntryRep lock=null;

			// segnalo allo spacevector che la operazione e' in corso
		pspace.blackboard.multipleOp(true);
		EntryRep[] arrayrep = pspace.rdAll(rep,txn,timeout,myId);
		//arrayrep[0].showValues();
		Entry[] array = new Entry[arrayrep.length];

		for (int i = 0; i < arrayrep.length ; i++) {
			array[i] = arrayrep[i].entry();
			arrayrep[i].unLock(myId);

		}

	 		// segnalo allo spacevector che la operazione e' finita
		pspace.blackboard.multipleOp(false);
	 
			// dico allo spacekernel di riorganizzare il vettore
		pspace.blackboard.reOrganize();
	 
			// risveglio gli agenti in attesa sulle tuple lockate
		pspace.notifing();	
		return array;
	} 

	/**********************************************
	 * deve cominciare la parte reattiva
	 ********************************************/
}


/**************************************************
 * take
 *************************************************/


public Entry take (Entry entry, Transaction txn, long timeout)
		throws NotInterfaceOwnerException,NoPermissionException 
{
		//si noti che questo metodo e' preaticamnete identico a
		//quello read, ma si usa la funzione in e non rd per
		//prelevare le tuple
	 	 
		// if (!myId.equals(who)) 
		//   throw new NotInterfaceOwnerException(); 

	long NO_WAIT = 0;
	
		//per il momento lascio queste come commento
	//if ( !acl_base.checkPermission(myId,new PermissionImpl("READ")) )
	//throw new NoPermissionException();

		// la parte reattiva si riduce a queste operazioni...
		// individuo tutte le reazioni che devono essere scatenate
		// tupla serializzata

	EntryRep rep = new EntryRep(entry);
	 
		//  tupla "loccata"
	EntryRep lock = null;
	 
		//  entry da ritornare
	Entry er = null; 
	
	//EntryRep errep; secondo me questa non serve a niente

	EntryRep resultrep = pspace.in(rep, txn,timeout, myId);
	lock = resultrep;
	Entry result = entry; //"partiamo" dal valore di entry e non da null
		//ovviamente per sfruttarlo nella def di reactionentry

	if (resultrep != null) 
		result = resultrep.entry();

	//System.out.println("oggetto");
	
		// a questo punto resul contiene o il template con cui abbiamo
		// fatto la read oppure il risultato della read, se questo non
		//e' null.
		//si rispettano dunque tutte le specifiche di MARS


	/****** PARTE REATTIVA DELLA TAKE ******/
	 	
	ReactionEntry reactionentry = new ReactionEntry(result,myId,"take",null);  

	Entry[] react;
	Entry tmp = null;
	Entry reactionpassa = null;

		//andiamo a vedere nello spazio di metalivello se ci sono tuple
		//corrispondenti in base alle quali scatenare delle reazioni
		//le mettiamo nell'array react
	
		/*
		System.out.println("ReactiveSpace: take: cerco reazioni nel metaspazio");
		System.out.println("uso il template:");
		reactionentry.showValues();
		*/

	react = mspace.readAll(reactionentry,txn,10000,myId);
 		//react = mspace.readAll(null,txn,NO_WAIT,null);

	if (react.length != 0) {

			//sono state trovare delle reazioni; per ognuna invochiamo
			//il TManager....


		for (int i=0; i<react.length;i++) {
		
				//ATTENZIONE: PER IL PRIMO PARAMETRO DA PASSARE A LAUNCH BISOGNA SELEZIONARE
				//IL CAMPO reaction DI react[i] e non react[i]

			try {
				Class classe= ((Object)react[i]).getClass();
				Field campo=classe.getField("reaction");
					//System.out.println("la reattivita' della take ha trovato la tupla");
					//react[i].showValues();
		 		tmp = TManager.Launch((ReactiveThread)campo.get(react[i]),reactionentry, 
				new Space(myId,pout,pspace), myId,null, reactionpassa);
			
			reactionpassa = tmp;	
				//}
				//System.out.println("Step 1");
			er = tmp; 
				//non ho capito perche' tmp viene assegnato sia 
				//a reactionpassa che a tmp

			}
			catch(NoSuchFieldException x) {System.out.println("no such field");}
			catch(IllegalAccessException y) {System.out.println("illegal");}

		}	
	 
			 // libero la tupla locked
		if (lock != null) 
			lock.unLock(myId);	 
	
			// risveglio gli agenti in attesa sulle tuple "loccate"
		pspace.notifing();

	}

	/*
	else 
		System.out.println("la reattivita' della take non ha trovato niente");
	*/

	return result;
}



/**********************************************
 * takeAll
 **********************************************/

public Entry[] takeAll(Entry entry, Transaction txn, long timeout)
		throws NotInterfaceOwnerException,NoPermissionException 
{
	long NO_WAIT = 0;
	
		//per il momento lascio queste come commento
	//if ( !acl_base.checkPermission(myId,new PermissionImpl("READ")) )
	//throw new NoPermissionException();
	
	if (entry == null) {
		Enumeration enum = pspace.blackboard.elements();
		Entry[] ritorna = new Entry[pspace.blackboard.size()];
		int i = 0;
		
		for(;enum.hasMoreElements();) {
			EntryRep list = (EntryRep)enum.nextElement();
			ritorna[i] = list.entry(false);
			i++;
		}
	
		return ritorna;
	}

	else {
		EntryRep rep = new EntryRep(entry);	 

		EntryRep lock=null;

			// segnalo allo spacevector che la operazione e' in corso
		pspace.blackboard.multipleOp(true);
	 
		EntryRep[] arrayrep = pspace.inAll(rep,txn,timeout,myId);
		Entry[] array = new Entry[arrayrep.length];
	 
		for (int i = 0; i < arrayrep.length ; i++) {
			array[i] = arrayrep[i].entry();
			arrayrep[i].unLock(myId);
		}

	 		// segnalo allo spacevector che la operazione e' finita
		pspace.blackboard.multipleOp(false);
	 
			// dico allo spacekernel di riorganizzare il vettore
		pspace.blackboard.reOrganize();
	 
			// risveglio gli agenti in attesa sulle tuple lockate
		pspace.notifing();	
		return array;
	} 

	/**********************************************
	 * qui deve cominciare la parte reattiva
	 ********************************************/
}


/*******************************************
 *	WRITE
 *******************************************/

public Lease write(Entry entry, Transaction txn, long lease) 
	throws NotInterfaceOwnerException, NoPermissionException
{
	
	if ( !pspace.checkPermission(myId,new PermissionImpl("REACTIVE")) )
		throw new NoPermissionException();

	if (!pspace.checkPermission(myId, new PermissionImpl("WRITE")) )
		throw new NoPermissionException();
	//Lease res=pspace.write(entry, txn, lease, myId);
	
	if (entry == null) {
		System.out.println("no argomento valido");
		return null;
	}
		
	else {
		EntryRep rep = new EntryRep(entry, false);

			// inserisco la tupla nello spazio
		Lease leaseentry = pspace.out(rep, txn,lease, myId);

			//if (leasentry != null) 
		long exp = rep.getExpiration();
		final long now = System.currentTimeMillis();
		long duration = exp - now;
  
			// "slocco" la tupla
		if (rep != null) 
			rep.unLock(myId);
	 
			// risveglio gli agenti in attesa sulle tuple lockate
		pspace.notifing();

		/****** PARTE REATTIVA DELLA WRITE ******/
		 
		Entry result=entry;
		Entry er=null;

		ReactionEntry reactionentry = new ReactionEntry(result,myId,"write",null);  

		Entry[] react;
		Entry tmp = null;
		Entry reactionpassa = null;
		EntryRep lock = null;

			//andiamo a vedere nello spazio di metalivello se ci sono tuple
			//corrispondenti in base alle quali scatenare delle reazioni
			//le mettiamo nell'array react


		react = mspace.readAll(reactionentry,txn,10000,myId);
 			//react = mspace.readAll(null,txn,NO_WAIT,null);

		if (react.length != 0) {

			//sono state trovare delle reazioni; per ognuna invochiamo
			//il TManager....


			for (int i=0; i<react.length;i++) {
		
				//ATTENZIONE: PER IL PRIMO PARAMETRO DA PASSARE A LAUNCH BISOGNA SELEZIONARE
				//IL CAMPO reaction DI react[i] e non react[i]

				try {
					//System.out.println("la reattivita' della read ha trovato la tupla");
					//react[i].showValues();
					Class classe= ((Object)react[i]).getClass();
					Field campo=classe.getField("reaction");
					//System.out.println("X"+yy.get(react[i]));

			 		tmp = TManager.Launch((ReactiveThread)campo.get(react[i]),reactionentry, 
					new Space(myId,pout,pspace), myId,null, reactionpassa);
			
					reactionpassa = tmp;	
						//}
						//System.out.println("Step 1");
					er = tmp; 
						//non ho capito perche' tmp viene assegnato sia 
						//a reactionpassa che a tmp

				}
				catch(NoSuchFieldException x) {System.out.println("no such field");}
				catch(IllegalAccessException y) {System.out.println("illegal");}

			}	
	 
				 // libero la tupla locked
			if (lock != null) 
				lock.unLock(myId);	 
	
				// risveglio gli agenti in attesa sulle tuple "loccate"
			pspace.notifing();


		}
		return leaseentry; 

	}
}
} //FINE DELLA CLASSE


