/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * CopyrightVersion 1.0.1_JCP 
 */
package net.jini.core.transaction;

import net.jini.core.transaction.server.*;
import net.jini.core.lease.LeaseDeniedException;
import java.rmi.RemoteException;

/**
 * Factory methods for creating top-level transactions.
 */
public class TransactionFactory {

    private TransactionFactory() {}

    /**
     * Create a new top-level transaction.  If the specified transaction
     * manager supports nested transactions, then the returned transaction
     * can be cast to a <code>NestableTransaction</code>.
     *
     * @param mgr the transaction manager to use for this transaction
     * @param leaseTime the requested lease time for the transaction
     * @return the created transaction and the lease granted
     */
    public static Transaction.Created create(TransactionManager mgr,
					     long leaseTime) 
        throws LeaseDeniedException, RemoteException
    {
        TransactionManager.Created rawTxn = mgr.create(leaseTime);
	Transaction transaction;
	if (mgr instanceof NestableTransactionManager)
	    transaction = new NestableServerTransaction(
					     (NestableTransactionManager)mgr,
					     rawTxn.id, null);
	else
	    transaction = new ServerTransaction(mgr, rawTxn.id);
        return new Transaction.Created(transaction, rawTxn.lease);
    }

    /**
     * Create a new top-level transaction, under which nested transactions
     * can be created.
     *
     * @param mgr the transaction manager to use for this transaction
     * @param leaseTime the requested lease time for the transaction
     * @return the created transaction and the lease granted
     */
    public static
    NestableTransaction.Created create(NestableTransactionManager mgr,
				       long leaseTime) 
        throws LeaseDeniedException, RemoteException
    {
        TransactionManager.Created rawTxn = mgr.create(leaseTime);
        return new NestableTransaction.Created(
				    new NestableServerTransaction(mgr,
								  rawTxn.id,
								  null),
				    rawTxn.lease);
    }
}
