/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.discovery;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * A holder class for constants that pertain to the unicast and
 * multicast discovery protocols.
 * 
 * @author Bryan O'Sullivan
 */
public class Constants {
    /**
     * The address of the multicast group over which the multicast
     * request protocol takes place.
     */
    private static InetAddress requestAddress = null;

    /**
     * The address of the multicast group over which the multicast
     * announcement protocol takes place.
     */
    private static InetAddress announcementAddress = null;

    /**
     * The port for both unicast and multicast boot requests.
     */
    public static final short discoveryPort = 4160;

    /**
     * This class cannot be instantiated.
     */
    private Constants() {}
  
    /**
     * Return the address of the multicast group over which the
     * multicast request protocol takes place.
     */
    public static final InetAddress getRequestAddress()
	throws UnknownHostException
    {
	if (requestAddress == null)
	    requestAddress = InetAddress.getByName("224.0.1.85");
	return requestAddress;
    }
  

    /**
     * Return the address of the multicast group over which the
     * multicast announcement protocol takes place.
     */
    public static final InetAddress getAnnouncementAddress()
	throws UnknownHostException
    {
	if (announcementAddress == null)
	    announcementAddress = InetAddress.getByName("224.0.1.84");
	return announcementAddress;
    }
}
