/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.lookup.entry;

import net.jini.entry.AbstractEntry;

/**
 * The address of the physical component of a service.  This is
 * distinct from the Location class in that it is intended for use
 * with the Location class in geographically dispersed organizations.
 *
 * @see Location
 * @see AddressBean
 * @author Bryan O'Sullivan
 */
public class Address extends AbstractEntry {
    private static final long serialVersionUID = 2896136903322046578L;

    /**
     * Construct an empty instance of this class.
     */
    public Address() {
    }

    /**
     * Construct an instance of this class, with all fields
     * initialized appropriately.
     */
    public Address(String street, String organization,
		   String organizationalUnit, String locality,
		   String stateOrProvince, String postalCode, String country)
    {
	this.street = street;
	this.organization = organization;
	this.organizationalUnit = organizationalUnit;
	this.locality = locality;
	this.stateOrProvince = stateOrProvince;
	this.postalCode = postalCode;
	this.country = country;
    }
    
    /**
     * Street address.  For example, "901&nbsp;San Antonio Road".
     *
     * @serial
     */
    public String street;

    /**
     * Name of the company or organization that provides this service.
     * For example, "Sun Microsystems".
     *
     * @serial
     */
    public String organization;

    /**
     * The unit within the organization that provides this service.
     * For example, "Information Services".
     *
     * @serial
     */
    public String organizationalUnit;

    /**
     * City or locality name.  For example, "Palo Alto".
     *
     * @serial
     */
    public String locality;

    /**
     * Full name or standard postal abbreviation of a state or
     * province.  For example, "CA" (for California).
     *
     * @serial
     */
    public String stateOrProvince;

    /**
     * Postal code.  For example, in the United States, this is a ZIP
     * code; in Ireland, it might be either empty or a postal district
     * of Dublin.
     *
     * @serial
     */
    public String postalCode;

    /**
     * Country name.
     *
     * @serial
     */
    public String country;
}
