/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.lookup.entry;

import net.jini.entry.AbstractEntry;

/**
 * The location of the physical component of a service.  This is
 * distinct from the Address class in that it can be used alone in a
 * small, local organization.
 *
 * @see Address
 * @see LocationBean
 * @author Bryan O'Sullivan
 */
public class Location extends AbstractEntry {
    private static final long serialVersionUID = -3275276677967431315L;

    /**
     * Construct an empty instance of this class.
     */
    public Location() {
    }

    /**
     * Construct an instance of this class, with all fields
     * initialized appropriately.
     */
    public Location(String floor, String room, String building) {
	this.floor = floor;
	this.room = room;
	this.building = building;
    }

    /**
     * A floor designation.  For example, "2".
     *
     * @serial
     */
    public String floor;

    /**
     * A room or cube number.  For example, "B250".
     *
     * @serial
     */
    public String room;

    /**
     * A building name or code.  For example, "SUN04".
     *
     * @serial
     */
    public String building;
}
