/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.lookup.entry;

import java.io.Serializable;
import net.jini.core.entry.Entry;

/**
 * A JavaBeans(TM) component that encapsulates a Location object.
 *
 * @see Location
 * @see Address
 * @see AddressBean
 * @author Bryan O'Sullivan
 */
public class LocationBean implements EntryBean, Serializable {
    private static final long serialVersionUID = -4182591284470292829L;

    /**
     * The Location object associated with this JavaBeans component.
     *
     * @serial
     */
    protected Location assoc;

    /**
     * Construct a new JavaBeans component linked to a new empty 
     * Location object.
     */
    public LocationBean() {
	assoc = new Location();
    }

    /**
     * Make a link to an Entry object.
     *
     * @param e the Entry object to link to
     * @exception java.lang.ClassCastException the Entry is not of the
     * correct type for this JavaBeans component
     */
    public void makeLink(Entry e) {
	assoc = (Location) e;
    }

    /**
     * Return the Entry linked to by this JavaBeans component.
     */
    public Entry followLink() {
	return assoc;
    }

    /**
     * Return the value of the floor field in the Location object linked to by
     * this JavaBeans component.
     */
    public String getFloor() {
	return assoc.floor;
    }

    /**
     * Set the value of the floor field in the Location object linked to by
     * this JavaBeans component.
     */
    public void setFloor(String x) {
	assoc.floor = x;
    }

    /**
     * Return the value of the room field in the Location object linked to by
     * this JavaBeans component.
     */
    public String getRoom() {
	return assoc.room;
    }

    /**
     * Set the value of the room field in the Location object linked to by this
     * JavaBeans component.
     */
    public void setRoom(String x) {
	assoc.room = x;
    }

    /**
     * Return the value of the building field in the Location object linked to
     * by this JavaBeans component.
     */
    public String getBuilding() {
	return assoc.building;
    }

    /**
     * Set the value of the building field in the Location object linked to by
     * this JavaBeans component.
     */
    public void setBuilding(String x) {
	assoc.building = x;
    }
}
