/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.lookup.entry;

import net.jini.entry.AbstractEntry;

/**
 * Generic information about a service.  This includes the name of the
 * manufacturer, the product, and the vendor.
 *
 * @see ServiceInfoBean
 * @author Bryan O'Sullivan
 */
public class ServiceInfo extends AbstractEntry implements ServiceControlled {
    private static final long serialVersionUID = -1116664185758541509L;

    /**
     * Construct an empty instance of this class.
     */
    public ServiceInfo() {
    }

    /**
     * Construct an instance of this class, with all fields
     * initialized appropriately.
     */
    public ServiceInfo(String name, String manufacturer, String vendor,
		       String version, String model, String serialNumber)
    {
	this.name = name;
	this.manufacturer = manufacturer;
	this.vendor = vendor;
	this.version = version;
	this.model = model;
	this.serialNumber = serialNumber;
    }

    /**
     * The name of the product or package of which this service is an
     * instance.  This field should not include the name of the
     * manufacturer or vendor.
     *
     * @serial
     */
    public String name;

    /**
     * The name of the manufacturer or author of this service.  For
     * example, "Sun Microsystems".
     *
     * @serial
     */
    public String manufacturer;

    /**
     * The name of the vendor of this service.  This may have the same
     * value as the manufacturer field, or it may be different.
     *
     * @serial
     */
    public String vendor;

    /**
     * The version of this service.  This is a free-form field, but
     * should follow accepted version-naming conventions to make
     * visual identification easier.
     *
     * @serial
     */
    public String version;

    /**
     * The model name or number of this service, if any.
     *
     * @serial
     */
    public String model;

    /**
     * The serial number of this instance of the service, if any.
     *
     * @serial
     */
    public String serialNumber;
}
