/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.lookup.entry;

import java.io.Serializable;

/**
 * Information pertaining to the severity of a Status object.
 *
 * @see Status
 * @author Bryan O'Sullivan
 */
public class StatusType implements Serializable {
    private static final long serialVersionUID = -8268735508512712203L;

    /** @serial */
    private final int type;

    private StatusType(int t) {
	type = t;
    }

    /**
     * An error has occurred that is preventing a service from
     * operating correctly.
     */
    public static final StatusType ERROR = new StatusType(1);

    /**
     * A problem merits attention, but is not impeding the operation
     * of the service.
     */
    public static final StatusType WARNING = new StatusType(2);

    /**
     * An unusual event is occurring, but the event is not necessarily
     * a problem.
     */
    public static final StatusType NOTICE = new StatusType(3);

    /**
     * A noteworthy event is occurring during the normal operation of
     * a service.
     */
    public static final StatusType NORMAL = new StatusType(4);
}
