/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.space;

import java.io.*;

/**
 * This exception denotes a problem with the local implementation of the
 * <code>JavaSpace</code> interface.  The <code>detail</code> field
 * will give a description that can be reported to the space developer
 * (and may be documented in that space's external documentation).
 *
 * @author Ken Arnold
 *
 * @see JavaSpace
 */
public class InternalSpaceException extends RuntimeException {
    static final long serialVersionUID = -4167507833172939849L;


    /**
     * The exception (if any) that trigged the internal exception.  This
     * may be <code>null</code>.
     *
     * @serial
     */
    public final Throwable nestedException;

    /**
     * Create an exception, forwarding a string to the superclass constructor.
     */
    public InternalSpaceException(String str) {
	super(str);
	nestedException = null;
    }

    /**
     * Create an exception, forwarding a string and exception to the
     * superclass constructor.
     */
    public InternalSpaceException(String str, Throwable ex) {
	super(str);
	nestedException = ex;
    }

    /**
     * Print the stack trace of this exception, plus that of the nested
     * exception, if any.
     */
    public void printStackTrace() {
	printStackTrace(System.err);
    }

    /**
     * Print the stack trace of this exception, plus that of the nested
     * exception, if any.
     */
    public void printStackTrace(PrintStream out) {
	super.printStackTrace(out);
	if (nestedException != null) {
	    out.println("nested exception:");
	    nestedException.printStackTrace(out);
	}
    }

    /**
     * Print the stack trace of this exception, plus that of the nested
     * exception, if any.
     */
    public void printStackTrace(PrintWriter out) {
	super.printStackTrace(out);
	if (nestedException != null) {
	    out.println("nested exception:");
	    nestedException.printStackTrace(out);
	}
    }
}
