/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.space;

import net.jini.core.entry.*;
import net.jini.entry.*;
import net.jini.core.transaction.*;
import net.jini.core.event.*;
import net.jini.core.lease.*;

import java.rmi.*;

/**
 * This interface is implemented by servers that export a
 * JavaSpaces(TM) technology service.  The operations in this interface
 * are the public methods that all such spaces support.
 *
 * @author Ken Arnold
 * @author Jim Waldo
 *
 * @see Entry
 */
public interface JavaSpace {
    /**
     * Write a new entry into the space.
     *
     * @exception TransactionException A transaction error occurred
     */
    //Lease write(Entry entry, Transaction txn, long lease)
	//throws TransactionException, SecurityException, RemoteException;

    /**
     * Wait for no time at all.  This is used as a timeout value in
     * various read and take calls.
     *
     * @see #read
     * @see #readIfExists
     * @see #take
     * @see #takeIfExists
     */
    long NO_WAIT = 0;

    /**
     * Read any matching entry from the space, blocking until one exists.
     * Return <code>null</code> if the timeout expires.
     *
     * @param tmpl      The template used for matching.  Matching is
     *			done against <code>tmpl</code> with blank
     *			fields being wildcards ("match anything") other
     *			fields being values ("match exactly on the
     *			serialized form").
     * @param txn	The transaction (if any) under which to work.
     * @param timeout	How long the client is willing to wait for a
     *			transactionally proper matching entry.  A
     *			timeout of <code>NO_WAIT</code> means to wait
     *			no time at all; this is equivalent to a wait
     *			of zero.
     *
     * @exception TransactionException A transaction error occurred
     */
    Entry read(Entry tmpl, Transaction txn, long timeout)
	throws UnusableEntryException, TransactionException, SecurityException,
	       InterruptedException, RemoteException;

    /**
     * Read any matching entry from the space, returning
     * <code>null</code> if there is currently is none.  Matching and
     * timeouts are done as in <code>read</code>, except that blocking
     * in this call is done only if necessary to wait for transactional
     * state to settle.
     *
     * @exception TransactionException A transaction error occurred
     * @see #read
     */
  //  Entry readIfExists(Entry tmpl, Transaction txn, long timeout)
	//throws UnusableEntryException, TransactionException, SecurityException,
	  //     InterruptedException, RemoteException;

    /**
     * Take a matching entry from the space, waiting until one exists.
     * Matching is and timeout done as for <code>read</code>.
     *
     * @see #read
     * @exception TransactionException A transaction error occurred
     */
    //Entry take(Entry tmpl, Transaction txn, long timeout)
	//throws UnusableEntryException, TransactionException, SecurityException,
	  //     InterruptedException, RemoteException;

    /**
     * Take a matching entry from the space, returning
     * <code>null</code> if there is currently is none.  Matching is
     * and timeout done as for <code>read</code>, except that blocking
     * in this call is done only if necessary to wait for transactional
     * state to settle.
     *
     * @see #read
     * @exception TransactionException A transaction error occurred
     */
   // Entry takeIfExists(Entry tmpl, Transaction txn, long timeout)
	//throws UnusableEntryException, TransactionException, SecurityException,
	  //     InterruptedException, RemoteException;

    /**
     * When entries are written that match this template notify the
     * given <code>listener</code> with a <code>RemoteEvent</code> that
     * includes the <code>handback</code> object.  Matching is done as
     * for <code>read</code>.
     *
     * @see #read
     * @see net.jini.core.event.EventRegistration
     */
    //EventRegistration
	//notify(Entry tmpl, Transaction txn, RemoteEventListener listener,
        //       long lease, Object handback)
	//throws TransactionException, SecurityException, RemoteException;

    /**
     * Return a snapshot of the given entry.  The snapshot can be used
     * anywhere the original entry can be, and will maintain the same value
     * as the original entry, even if the entry is modified after the
     * snapshot is taken.  The implementation of the
     * <code>JavaSpace</code> interface the snapshot more efficient to
     * use than repeated uses of the original entry.
     * <p>
     * Using a snapshot with a different space than the one that
     * produced it yields unpredictable results.
     */
    //Entry snapshot(Entry e) throws RemoteException;
}
